/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package psp.timelog;

import ccl.util.Util;
import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.border.Border;

import psp.timelog.util.ObservableValue;

/**
 * Encapsulates a combo box.
 *
 * @author Chr. Clemens Lee (clemens@kclee.com)                 *
 *         , Kenneth Westelinck (kenneth.westelinck@pandora.be) *
 * @version $Id: ChoiceObserver.java,v 1.2 2003/05/03 12:13:00 c1emens Exp $
 */
public class ChoiceObserver extends JPanel
                            implements Observer
                                       , ActionListener 
{
    private JComboBox processComboBox = new JComboBox();
    private ObservableValue _oblvChoice = null;

    public ChoiceObserver( ObservableValue oblvChoice_
                           , Enumeration eElements_ )
    {
        super();

        Util.debug("ChoiceObserver: <init>: eElements_: " + eElements_);

        _oblvChoice = oblvChoice_;

        /*processComboBox = getComboboxForEnumeration(eElements_);*/
        updateItems( eElements_ );
        processComboBox.addActionListener(this);

        setLayout(new BorderLayout());
        add(processComboBox);
    }

    public void update(Observable pObservable_, Object pObject_) 
    {
        processComboBox.setSelectedIndex(
            Integer.parseInt(_oblvChoice.toString()));
    }

    static private JComboBox getComboboxForEnumeration( Enumeration e )
    {
        Vector processes = new Vector();
        if (e != null) 
        {
            for (; e.hasMoreElements();) 
            {
                processes.addElement((String) e.nextElement());
            }
        }

        return new JComboBox(processes);
    }

    public void updateItems( Enumeration eItems_ )
    {
	processComboBox.removeAllItems();
        if (eItems_ != null) {
            for (; eItems_.hasMoreElements();) {
                processComboBox.addItem((String) eItems_.nextElement());
            }
        }
        _oblvChoice.set("0");
    }

    public void actionPerformed( ActionEvent e )
    {
        JComboBox cb = (JComboBox) e.getSource();
        _oblvChoice.set(String.valueOf(cb.getSelectedIndex()));
    }

    public void setFont( Font font )
    {
        if ( processComboBox != null )
        {
            processComboBox.setFont( font );
        }
    }

    /**
     * Sets border on encapsulated combo box.
     *
     * @see javax.swing.JComponent#setBorder(javax.swing.border.Border)
     */
    public void setBorder(Border border) 
    {
        if ( processComboBox != null )
        {
            //((JComponent) processComboBox.getEditor().getEditorComponent()).setBorder(border);
            ((JComponent) processComboBox.getComponent( 0 )).setBorder( border);
        }
    }

}
