/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package psp.timelog;

import ccl.swing.AutoGridBagLayout;
import ccl.swing.InputCancelDialog;
import ccl.swing.ListCancelSelector;
import ccl.swing.ListEditDialog;
import ccl.swing.MainJFrame;
import ccl.swing.SwingUtil;
import ccl.util.FileUtil;
import ccl.util.Util;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Date;
import java.util.Enumeration;
import java.util.Vector;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.border.Border;
import psp.timelog.swing.StatusBar;
import psp.timelog.util.ObservableValue;

/**
 * @author Chr. Clemens Lee (clemens@kclee.com)                 *
 *         , Kenneth Westelinck (kenneth.westelinck@pandora.be) *
 * @version $Id: Controller.java,v 1.3 2003/05/03 12:13:00 c1emens Exp $
 */
public class Controller extends MainJFrame
                        implements Controllable,
                                   ActionListener
{
    static private final Dimension CONTENT_SIZE = new Dimension( 677, 528 - 29 ); // 677, 528 - 29 );// 695x565, 728, 582
    static private final Insets BUTTON_MARGIN = new Insets( 0, 0, 0, 0 );//0, 3, 0, 3 );
    static private final int VERTICAL_STRUT = 10;
    static private final Font FONT = new Font( "Dialog", Font.PLAIN, 12 );
    /*
    static private final Border buttonBorder = BorderFactory.createCompoundBorder( SwingUtil.createCCLButtonBorder()
                                                                                   , BorderFactory.createEmptyBorder( 0, 0, 2, 0 ) );
										   // */
    static private Border buttonBorder = SwingUtil.createCCLButtonBorder();

    private static final String S_NEW = "<new>";

    private SessionView _pSessionView;
    private Session     _pSession;
    private Timelog     _pTimelog;
    private TimelogView _pTimelogView;
    private SummaryView _pSummaryView;
    private JButton     _btnAdd;
    private JButton     _btnChange;
    private JButton     _btnDelete;
    private StatusBar   _statusBar = null;

    private boolean _bModified = false;

    private String _sProcess = "";
    private Vector _vProcess = new Vector();

    private static final String S_MODIFIED = "Mod";

    private static final String S_MNU_FILE_EXPORT = "Export...";

    private static final int MNU_FILE = 0;
    private static final int MNU_FILE_OPEN = 1;
    private static final int MNU_FILE_SAVE = 2;
    private static final int MNU_FILE_PROCESS = 3;
    private static final int MNU_FILE_EXIT = 4;
    private static final int MNU_HELP = 5;
    private static final int MNU_HELP_CONTENTS = 6;
    private static final int MNU_HELP_ABOUT = 7;
    private static final int MNU_NUMBER = 8;

    private String[][] _aasMenuString = new String[MNU_NUMBER][2];

    private static final int LIST_SPACE = 2;


    /**
     * Returns standard font used for buttons and labels.
     */
    static public Font getStandardFont()
    {
        return FONT;
    }

    /**
     * Sets the standard font on the given swing component.
     */
    static public void setFontOn( JComponent component )
    {
        component.setFont( getStandardFont() );
    }

    /**
     * Sets the standard button border on the given swing component.
     */
    public void setBorderOn( JComponent component )
    {
	if ( buttonBorder != null )
	{
	    if ( useWindowsButtons() == false )
	    {
		buttonBorder = null;
	    }
	    else
	    {
		component.setBorder( buttonBorder );
	    }
	}
    }

    /**
     * Insets used as margin in buttons.
     */
    static public Insets getButtonMargin()
    {
        return BUTTON_MARGIN;
    }

    /**
     * Vertical strut used uniformly for empty lines.
     */
    static public Component createVerticalStrut()
    {
        return Box.createVerticalStrut( VERTICAL_STRUT );
    }

    private void _setTitle()
    {
        String sTitle = getInit().getApplicationName() 
               + ": " 
               + getInit().getFileName();
        setTitle(sTitle);
    }

    private void _setMenuBar()
    {
        _aasMenuString[MNU_FILE][0] = "File";
        _aasMenuString[MNU_FILE][1] = "Datei";
        _aasMenuString[MNU_FILE_OPEN][0] = "Open...";
        _aasMenuString[MNU_FILE_OPEN][1] = "ffnen...";
        _aasMenuString[MNU_FILE_SAVE][0] = "Save";
        _aasMenuString[MNU_FILE_SAVE][1] = "Speichern";
        _aasMenuString[MNU_FILE_PROCESS][0] = "Processes...";
        _aasMenuString[MNU_FILE_PROCESS][1] = "Prozesse...";
        _aasMenuString[MNU_FILE_EXIT][0] = "Exit";
        _aasMenuString[MNU_FILE_EXIT][1] = "Beenden";
        _aasMenuString[MNU_HELP][0] = "Help";
        _aasMenuString[MNU_HELP][1] = "Hilfe";
        _aasMenuString[MNU_HELP_CONTENTS][0] = "Contents...";
        _aasMenuString[MNU_HELP_CONTENTS][1] = "Inhalt...";
        _aasMenuString[MNU_HELP_ABOUT][0] = "About...";
        _aasMenuString[MNU_HELP_ABOUT][1] = "Info...";

        int language = 0;

        Vector vMenus = new Vector();
        Vector vMenuFile = new Vector();
        Vector vMenuHelp = new Vector();

        vMenuFile.addElement((Object) _aasMenuString[MNU_FILE][language]);
        vMenuFile.addElement((Object) _aasMenuString[MNU_FILE_OPEN][language]);
        vMenuFile.addElement((Object) _aasMenuString[MNU_FILE_SAVE][language]);
        vMenuFile.addElement(S_MNU_FILE_EXPORT);
        vMenuFile.addElement(
            (Object) _aasMenuString[MNU_FILE_PROCESS][language]);
        vMenuFile.addElement((Object) _aasMenuString[MNU_FILE_EXIT][language]);

        // Help
        vMenuHelp.addElement(_aasMenuString[MNU_HELP][language]);
        Util.debug("Controller: Controller: vMenuHelp: " + vMenuHelp);
        /*vMenuHelp.addElement(_aasMenuString[MNU_HELP_CONTENTS][language]);
        vMenuHelp.addElement("--------");*/
        vMenuHelp.addElement((Object) _aasMenuString[MNU_HELP_ABOUT][language]);
        Util.debug(
            "Controller: Controller: _aasMenuString[MNU_HELP_ABOUT][language]: "
                + _aasMenuString[MNU_HELP_ABOUT][language]);
        Util.debug("Controller: Controller: vMenuHelp: " + vMenuHelp);

        vMenus.addElement((Object) vMenuFile);
        vMenus.addElement((Object) vMenuHelp);
        Util.debug("Controller: Controller: vMenuFile: " + vMenuFile);
        Util.debug("Controller: Controller: vMenuHelp: " + vMenuHelp);

        setMenuBar(vMenus);
    }

    private String _selectProcess() {
        // collect all processes in vector
        String sProcesses =
            getInit().getKeyValue(S_INIT_PROCESS, S_INIT_PROC_PROCESSES);
        Vector vProcesses = Util.stringToLines(sProcesses, C_PROCESS_DELIMITER);
        Util.panicIf(vProcesses == null || vProcesses.size() == 0);

	String sSelectedProcess = SwingUtil.inputListCancel(this, "Please select a work process!", vProcesses);
        Util.debug( "Controller: _getProcess: sSelectedProcess: " + sSelectedProcess);

        return sSelectedProcess;
    }

    private void _clearModified() {
        Util.debug("Controller._clearModified()");
        _bModified = false;
        _statusBar.setText( " " );
    }

    public void setModified() {
        Util.debug("Controller.setModified()");
        _bModified = true;
        _statusBar.setText( S_MODIFIED );
    }

    private boolean _isQuitGuardOk() {
        if (_bModified == false) {
            return true;
        }
        int yesNoOrCancel = SwingUtil.wannaSave(this);

        if (yesNoOrCancel == SwingUtil.YES) {
            _pTimelog.save();

            return true;
        }

        return (yesNoOrCancel != SwingUtil.CANCEL);
    }

    private Vector _getProcesses() {
        String sProcesses =
            getInit().getKeyValue(S_INIT_PROCESS, S_INIT_PROC_PROCESSES);
        Vector vProcesses = Util.stringToLines(sProcesses, C_PROCESS_DELIMITER);

        return vProcesses;
    }

    public Controller() {
        super();
    }

    public void initialize( String[] asArg_ )
    {
        initialize( asArg_, null );
    }

    public void initialize( String[] asArg_, String sInfoHeader_ )
    {
        if (Util.isEmpty(sInfoHeader_)) 
	{
            sInfoHeader_ = Main.S_RCS_HEADER;
        }
        super.initialize(asArg_, sInfoHeader_);

        if (isExitSet()) {
            return;
        }


        JPanel contentPane = (JPanel) this.getContentPane();
        contentPane.setBorder( BorderFactory.createEmptyBorder( 6, 6, 5, 6 ) );
        contentPane.setPreferredSize( CONTENT_SIZE );

        AutoGridBagLayout pAutoGridBagLayout = new AutoGridBagLayout();
        contentPane.setLayout(pAutoGridBagLayout);
        pAutoGridBagLayout.setInsets( 4, 2 );

        String sFileName = "";
        if (getInit().getArguments().size() != 0) {
            sFileName = (String) getInit().getArguments().firstElement();
        }
        if (!Util.isEmpty(sFileName)) {
            getInit().setFileFullName(sFileName);
        }

        _pSession = new Session(getInit());
        _pTimelog = new Timelog( this, _pSession );
        _pTimelogView = new TimelogView(_pTimelog, this);

        pAutoGridBagLayout.setExpandBoth();
        //pAutoGridBagLayout.setExpandBoth( 1.0, 300.0 );
        pAutoGridBagLayout.setFillBoth();
        pAutoGridBagLayout.setExtend(2, 1);
        contentPane.add(_pTimelogView);
        pAutoGridBagLayout.setExtend(1, 1);
        pAutoGridBagLayout.setFillNone();
        pAutoGridBagLayout.setExpandNone();
        pAutoGridBagLayout.endLine();
        
        _pSummaryView = new SummaryView(this, _pTimelog);

        //_pSummaryView.pack();
        //_pSummaryView.setMinimumSize( new Dimension( 200, 200 ) );
        _pSummaryView.setMinimumSize( new Dimension( _pSummaryView.getPreferredSize().width, 0 ) );

        pAutoGridBagLayout.setFillBoth();
        pAutoGridBagLayout.setExtend(1, 5);
        pAutoGridBagLayout.setAnchor(AutoGridBagLayout.SOUTHWEST);
        contentPane.add(_pSummaryView);
        pAutoGridBagLayout.setAnchor(AutoGridBagLayout.CENTER);
        pAutoGridBagLayout.setExtend(1, 1);

        _pSessionView = new SessionView(_pSession, this, _pTimelog);

        pAutoGridBagLayout.setAnchor(AutoGridBagLayout.NORTHEAST);
        pAutoGridBagLayout.setExpandNone();
        pAutoGridBagLayout.setFillNone();
        contentPane.add(_pSessionView);
        pAutoGridBagLayout.setAnchor(AutoGridBagLayout.CENTER);
        /*Dimension dimSummaryView = _pSummaryView.getSize();*/
        pAutoGridBagLayout.endLine();

        JPanel buttonPanel = new JPanel();
        buttonPanel.setBorder( BorderFactory.createEmptyBorder( 0, 0, 0, 2 ) );
        buttonPanel.setLayout( new GridLayout( 1, 3, 4, 0 ) );
        _btnAdd = new JButton("Add");
        _btnChange = new JButton("Change");
        _btnDelete = new JButton("Delete");

        setFontOn( _btnAdd );
        setFontOn( _btnChange );
        setFontOn( _btnDelete );

        setBorderOn( _btnAdd );
        setBorderOn( _btnChange );
        setBorderOn( _btnDelete );

        _btnAdd   .setMinimumSize( _pSessionView.getPreferredButtonSize() );
        _btnChange.setMinimumSize( _pSessionView.getPreferredButtonSize() );
        _btnDelete.setMinimumSize( _pSessionView.getPreferredButtonSize() );

        _btnAdd.setMargin( getButtonMargin() );
        _btnChange.setMargin( getButtonMargin() );
        _btnDelete.setMargin( getButtonMargin() );

        _btnAdd.addActionListener(this);
        _btnChange.addActionListener(this);
        _btnDelete.addActionListener(this);
        buttonPanel.add( _btnAdd );
        buttonPanel.add( _btnChange );
        buttonPanel.add( _btnDelete );

        // empty line
        pAutoGridBagLayout.skip();
        pAutoGridBagLayout.setFillBoth();
        contentPane.add( createVerticalStrut() );
        pAutoGridBagLayout.setFillNone();
        pAutoGridBagLayout.setExpandNone();
        pAutoGridBagLayout.endLine();

        pAutoGridBagLayout.skip();
        pAutoGridBagLayout.setAnchor(AutoGridBagLayout.EAST);
        contentPane.add( buttonPanel );
        pAutoGridBagLayout.endLine();

        // empty line
        pAutoGridBagLayout.skip();
        pAutoGridBagLayout.setFillBoth();
        contentPane.add( createVerticalStrut() );
        pAutoGridBagLayout.setFillNone();
        pAutoGridBagLayout.setExpandNone();
        pAutoGridBagLayout.endLine();

        /*_lblStatusBar = new JLabel( " " );
        _lblStatusBar.setFont( getStandardFont() );
        _lblStatusBar.setBorder( SwingUtil.createCCLBorder() );*/
	_statusBar = new StatusBar();
        _statusBar.setMinimumSize( _pSessionView.getPreferredButtonSize() );

        pAutoGridBagLayout.skip();
        pAutoGridBagLayout.setFillBoth();
        pAutoGridBagLayout.setExpandNone();
        pAutoGridBagLayout.setAnchor(AutoGridBagLayout.SOUTH);
        /*_statusBar.setSize( new Dimension( 0, 120 ) );*/
        contentPane.add( _statusBar );
        /*contentPane.add( _lblStatusBar );*/
        pAutoGridBagLayout.setFillNone();


        Util.debug(
            "Controller: Controller: _pTimelogView.size(): "
                + _pTimelogView.getSize());

        _setTitle();
        _setMenuBar();

        pack();

        
        /* Center the window. */
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension frameSize = getSize();
        if (frameSize.height > screenSize.height) {
            frameSize.height = screenSize.height;
        }
        if (frameSize.width > screenSize.width) {
            frameSize.width = screenSize.width;
        }
        setLocation(
            (screenSize.width - frameSize.width) / 2,
            (screenSize.height - frameSize.height) / 2);

        setVisible(true);

        Util.debug(
            "Controller: Controller: _pTimelogView.size(): "
                + _pTimelogView.getSize());
        Util.debug(
            "Controller: Controller: _pTimelogView.minimumSize(): "
            + _pTimelogView.getMinimumSize());

        Util.debug("Controller: Controller: date(): " + (new Date()));

        init();
    }

    /**
     * @param asArg_ the original String[] value from a static 
     *               main method. But it must be at least of 
     *               length 1. In case * it isn't, set it to "".
     */
    public Controller(String[] asArg_, String sInfoHeader_) {
        super();

        initialize(asArg_, sInfoHeader_);
    }
    
    /*
    public Insets insets() {
        return this.getInsets();
    }

    public Insets getInsets() {
        Insets inThis = super.insets();
        return inThis;
    }
    */

    public void init() {
        open(getInit().getFileFullName());

        _setTitle();
        updateMenu();
    }

    private boolean _bStartup = true;

    /**
     * Open specified timelog project.
     */
    public void open(String sFileFullName_) {
	open( sFileFullName_, null );
    }

    /**
     * Open specified timelog project.
     *
     * @param   sProcessForNewProject   used only if set for new project, when
     *                                  project file doesn't exist yet. Otherwise
     *                                  interactive dialog gets opened.
     *                                  It is used primarily for testing purposes.
     */
    public void open(String sFileFullName_
		     , String sProcessForNewProject )
    {
        if (!_isQuitGuardOk()) {
            return;
        }

        // Create a new file or open an old file.
        if (Util.isEmpty(sFileFullName_)) {
            String sFileSuffix = getInit().getKeyValue("Suffix");
            sFileFullName_ =
                SwingUtil.getFileName(
                    this,
                    "Choose a project",
                    "*" + sFileSuffix,
                    ".");

            Util.debug("Controller: open: sFileFullName_: " + sFileFullName_);
            if (Util.isEmpty(sFileFullName_)) {
                // didn't select any file
                if (Util.isEmpty(getInit().getFileFullName())) {
                    SwingUtil.showMessage(this, "No Project File specified.");
                    if (_bStartup) {
                        setExit();
                    }
                }

                return;
            }
            if ((!Util.endsWith(sFileFullName_, sFileSuffix))) {
                // has to end with .timelog
                SwingUtil.showMessage(
                    this,
                    "Error: Project file must end with '" + sFileSuffix + "'!");
                if (_bStartup) {
                    setExit();
                }

                return;
            }
            if (sFileFullName_.indexOf('*') != -1) {
                // no '*' allowed in project name
                SwingUtil.showMessage(
                    this,
                    "Error: No '*' allowed in project name!");
                if (_bStartup) {
                    setExit();
                }

                return;
            }
        }
        getInit().setFileFullName(sFileFullName_);
        _bStartup = false;

        _setTitle();
        updateMenu();

        // If the file does not exist, ask for process work and create the file.
        if ( !FileUtil.existsFile( sFileFullName_ ) )
        {
	    if ( sProcessForNewProject != null )
	    {
		_sProcess = sProcessForNewProject;
	    }
	    else
	    {
		_sProcess = _selectProcess();
	    }

            if ( _sProcess == null )
            {
                // no process selected, return
                return;
            }

            // create new .timelog file
            try {
                FileUtil.writeFile(
                    getInit().getFileFullName(),
                    _sProcess + "\n");
            } catch (Exception e) {
                SwingUtil.showMessage(
                    this,
                    "Error: Data File could not be saved!");
            }
        }

        // Load the timelog file.
        String sTimelog = "";
        try {
            Util.debug(
                "Controller: open: getInit().getFileFullName(): "
                    + getInit().getFileFullName());
            sTimelog = FileUtil.readFile(getInit().getFileFullName());
        } catch (Exception e) {
            SwingUtil.showMessage(
                this,
                "The Project File could not be opened!");
            // zu diesem zeitpunkt fand noch kein start() statt.
            setExit();

            return;
        }


        // Extract the process work.
        int index = sTimelog.indexOf('\n');
        if (index == -1) {
            SwingUtil.showMessage(this, "Error in Timelog File!");
            exit();
        }


        // A timelog file exists, but a process work was not selected. Repeat
        // showing the selection until a process is selected.
        _sProcess = sTimelog.substring(0, index);
        while ((_sProcess == null) || (_sProcess.length() == 0) ||
                _sProcess.equals("null"))
        {
            _sProcess = _selectProcess();
        }

        // Get the process elements from the ini file. If the elements are not
        // defined in the ini file, ask for a new process work.
        String sProcessElements =
            getInit().getKeyValue(S_INIT_PROCESS, _sProcess);
        // Repeat until a process is found which exists in the ini file
        // This could be quite confusing for the user, but it is still
        // better than to bomb.
        while ((sProcessElements == null) || (sProcessElements.length() == 0))
        {
            _sProcess = _selectProcess();
            sProcessElements = getInit().getKeyValue(S_INIT_PROCESS, _sProcess);
        }

        _vProcess = Util.stringToLines(sProcessElements, C_PROCESS_DELIMITER);
        Util.debug("Controller: open: _vProcess: " + _vProcess);

        // update Phase/Choice
        _pSessionView.updatePhaseChoice();
        _setTitle();

        // load data (old data gets automatically deleted)
        sTimelog = sTimelog.substring(index + 1, sTimelog.length());
        _pTimelog.load(sTimelog);
        _clearModified();
    }

    public void openOldProject(String sOldProjectName_) {
        String sOldProjectFullName = null;

        sOldProjectFullName = getInit().getOldFileFullName(sOldProjectName_);
        Util.debug("Controller: sOldProjektFullName: " + sOldProjectFullName);

        Util.panicIf(sOldProjectFullName == null, "Controller: openOldProject");

        open(sOldProjectFullName);
    }

    public Enumeration getProcessElements() {
        Util.debug("Controller: getProcessElements: _vProcess: " + _vProcess);
        return _vProcess.elements();
    }

    /**
     * Returns a descriptive string for a process id.
     */
    public String getProcessElement(String sIndex_) {
        int index = Util.atoi(sIndex_);
        Util.panicIf(index >= _vProcess.size());

        return ((String) _vProcess.elementAt(index));
    }

    public int getProcessSize() {
        return _vProcess.size();
    }

    public String getProcess() {
        return _sProcess;
    }

    class ProcessEdit {
        public ProcessEdit() 
	{
            // get existing processes
            Vector vProcesses = _getProcesses();
            vProcesses.insertElementAt(S_NEW, 0);
            Util.debug(
                "Controller.ProcessEdit.run().vProcesses: " + vProcesses);

            // select a Process
            String sProcessName = SwingUtil.inputListCancel( Controller.this
							     , "Please select a process to edit!"
							     , vProcesses );
            if ( Util.isEmpty( sProcessName ) )
	    {
                return;
            }
            vProcesses.removeElementAt(0);

            // New Process?
            if (sProcessName.equals(S_NEW)) {
                sProcessName = SwingUtil.inputCancel
		    ( Controller.this
		      , "New Process Name:"
		      , "");
                if ( sProcessName == null )
		{
                    return;
                }
                sProcessName = sProcessName.trim();
                if (Util.isEmpty(sProcessName)
                    || vProcesses.contains(sProcessName)) {
                    return;
                }
                vProcesses.addElement(sProcessName);
            }

            // get process elements
            String sProcessElements =
                getInit().getKeyValue(S_INIT_PROCESS, sProcessName);
            Vector vProcessElements = new Vector();
            if (!Util.isEmpty(sProcessElements)) {
                vProcessElements =
                    Util.stringToLines(sProcessElements, C_PROCESS_DELIMITER);
            }

            // Item List
            ListEditDialog pListEditDialog =
                new ListEditDialog(
                    Controller.this,
                    "Edit Elements of the " + sProcessName + " Process",
                    vProcessElements);
            Util.debug("Controller.ProcessEdit.run().1");
            pListEditDialog.show();
            if (!pListEditDialog.isOK()) {
                return;
            }
            vProcessElements = pListEditDialog.getValue();

            // Ask user if process is correct
            // ach was solls, lassen wir das heute weg ;-)

            // save process elements
            sProcessElements =
                Util.concat(vProcessElements, C_PROCESS_DELIMITER);
            boolean bError =
                getInit().setKeyValue(
                    S_INIT_PROCESS,
                    sProcessName,
                    sProcessElements);
            if (bError) {
                SwingUtil.showMessage(
                    Controller.this,
                    "Something went wrong\nwhile trying to write\n the new process elements to disk!");
            }
            // save processes
            String sProcesses = Util.concat(vProcesses, C_PROCESS_DELIMITER);
            bError =
                getInit().setKeyValue(
                    S_INIT_PROCESS,
                    S_INIT_PROC_PROCESSES,
                    sProcesses);
            if (bError) {
                SwingUtil.showMessage(
                    Controller.this,
                    "Something went wrong\nwhile trying to write\n the new process elements to disk!");
            }
        }
    }

    /*private Runnable _runMenu = null;*/

    public void actionPerformed(ActionEvent pActionEvent_) {
        Object oSource = pActionEvent_.getSource();
        if (oSource == _btnAdd) {
            _pSessionView.updateSession();
            _pTimelog.addSession(_pSession.toString());
        } else if (oSource == _btnDelete) {
            _pSessionView.updateSession(); // schadet ja nichts
            _pTimelog.deleteSession();
        } else if (oSource == _btnChange) {
            _pSessionView.updateSession();
            _pTimelog.changeSession();
        } else if (
            oSource instanceof JButton
                && ((JButton) oSource).getText().equals("Pause")) {
            _pSessionView.pause();
        } else if (
            oSource instanceof JButton
                && ((JButton) oSource).getText().equals(
                    SessionView.S_LAST_COMMENT)) {
            String sComment = _pTimelog.getLastComment();
            ObservableValue oblvComment = _pSession.getComment();
            oblvComment.set(sComment);
        } else if (oSource instanceof JMenuItem) {
            String sMenuItem = ((JMenuItem) oSource).getText();
            if ('1' <= sMenuItem.charAt(0) && sMenuItem.charAt(0) <= '9') {
                /*openOldProject(
                    sMenuItem.substring(
                        sMenuItem.indexOf(' ') + 1,
                        sMenuItem.length()));*/
            } else {
                int language = 0;
                if (((JMenuItem) oSource)
                    .getText()
                    .equals(_aasMenuString[MNU_FILE_SAVE][language])) {
                    _pTimelog.save();
                    _clearModified();
                } else if (
                    ((JMenuItem) oSource).getText().equals(
                        _aasMenuString[MNU_FILE_OPEN][language])) {
                    open("");
                } else if (
                    ((JMenuItem) oSource).getText().equals(
                        _aasMenuString[MNU_FILE_PROCESS][language])) {
                    ProcessEdit p = new ProcessEdit();
                    //interrupt();
                } else if (
                    ((JMenuItem) oSource).getText().equals(
                        S_MNU_FILE_EXPORT)) {
                    _pTimelog.export();
                }
            }
        }
        super.actionPerformed(pActionEvent_);
    }

    public boolean exit() {
        if (!_isQuitGuardOk()) {
            return false;
        }

        return super.exit();
    }

    /**
     * Export timelog object so it can be used for testing purposes.
     */
    public Timelog getTimelog() {
        return _pTimelog;
    }

    /**
     * Returns the summary view for testing purposes.
     */
    public SummaryView getSummaryView()
    {
        return _pSummaryView;
    }

    /**
     * Checks the application ini file to see if Windows look alike buttons should
     * be used. If nothing is specified, the answer will be true.
     */
    public boolean useWindowsButtons()
    {
	String sUse = getInit().getKeyValue( "UseWindowsButtons" );
	if ( Util.isEmpty( sUse ) )
	{
	    sUse = "true";
	}

	return Util.atob( sUse.trim() );
    }
}
