/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package psp.timelog;

import ccl.util.ClassPathUtil;
import ccl.util.Util;
import com.l2fprod.gui.plaf.skin.Skin;
import com.l2fprod.gui.plaf.skin.SkinLookAndFeel;
import java.io.File;
import javax.swing.UIManager;

/****************************************************************\
 * Program: Timelog                                             *
 *                                                              *
 * @author Chr. Clemens Lee (clemens@kclee.com)                 *
 *         , Kenneth Westelinck (kenneth.westelinck@pandora.be) *
 * @version $Id: Main.java,v 1.5 2003/05/04 12:23:06 c1emens Exp $
 *          , Date: 3. 9. 1996                                  *
\****************************************************************/
public class Main {
    private static final boolean B_DEBUG = false;
    
    // hack for version numbering until fixed when moved to swing
    private static final String VERSION = "8.25";
    private static final String DATE = "2003/05/04 19:55:00";
    public  static final String S_RCS_HEADER = "$" + "Header: /cvsroot/time1og/timelog/src/psp/timelog/RCS/Main.java,v " + VERSION + " " + DATE + " c1emens Exp $";

    /**
     * Constructor starts application and waits until it is finished.
     * If selected it also installs the SkinLF using the Aqua look.
     */
    public Main( String[] asArg_ )
    {
        super();

	boolean noAqua = false;
	for( int argument = 0; argument < asArg_.length; argument ++ )
	{
	    if ( asArg_[ argument ].equals( "-noaqua" ) )
	    {
		noAqua = true;
	    }
	}
	if ( noAqua == false )
	{
	    try
	    {
		String aquaFile = ClassPathUtil.getJarFileName( "com.l2fprod.gui.plaf.skin.SkinLookAndFeel" );
		aquaFile = aquaFile.substring( 0, aquaFile.lastIndexOf( File.separatorChar ) ) + File.separator + "aquathemepack.zip";
		Skin skin = SkinLookAndFeel.loadThemePack( aquaFile );
		SkinLookAndFeel.setSkin( skin );
		UIManager.setLookAndFeel( "com.l2fprod.gui.plaf.skin.SkinLookAndFeel" );
	    }
	    catch( Exception exception )
	    {
		Util.printlnErr( "Warning: unable to apply aqua skin look and feel : " + exception );
	    }
	}

	Controller pController = new Controller( asArg_, S_RCS_HEADER );
	pController.start();
    }
   
    /**
     * Main method of Timelog application.
     */
    public static void main( String[] asArg_ )
    {
        Util.setDebug(B_DEBUG);

        Main pMain = new Main(asArg_);

        System.exit( 0 );
    }
}
