/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package psp.timelog;

import javax.swing.JFrame;
import java.awt.GridLayout;
import javax.swing.JLabel;
import javax.swing.JTextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Calendar;
import java.util.Date;

import javax.swing.JButton;

import ccl.swing.ExitJDialog;
import ccl.swing.SwingUtil;
import ccl.util.Init;
import ccl.util.Util;

/**
 * @author <a href="http://www.kclee.com/clemens/">
 *         Chr. Clemens Lee</a>
 *         (<a href="mailto:clemens@kclee.com">
 *         clemens@kclee.com
 *         </a>)
 * @version $Id: PauseDialog.java,v 1.2 2003/05/03 12:13:00 c1emens Exp $
 */
public class PauseDialog extends ExitJDialog implements Runnable, ActionListener {
    public static final String S_TITLE = "Pause";

    private Calendar _clnStart = null;
    private Calendar _clnNull = null;
    private Calendar _clnOldPause = null;

    private boolean _bOk = false;
    private JButton _btnOk = new JButton("OK");
    private JButton _btnCancel = new JButton("Cancel");
    
    private JTextField _txtStart = new JTextField("", 5);
    private JTextField _txtDuration = new JTextField("", 5);
    private JTextField _txtTotal = new JTextField("", 5);

    private Thread _pThread;

    private void _showTime() {
        // offset stuff
        Date dtNow = new Date();
        long lNow = dtNow.getTime();
        long lStart = _clnStart.getTime().getTime();
        Util.debug( this, "_showTime().lNow:   " + lNow );
        Util.debug( this, "_showTime().lStart: " + lStart );

        // this interrupt
        long lDuration = lNow - lStart;
        Calendar clnDuration = (Calendar)_clnNull.clone();
        clnDuration.setTime( new Date( _clnNull.getTime().getTime() + lDuration ) );
        String sDuration = Util.getTime( clnDuration );
        Util.debug( this, "_showTime().sDuration: " + sDuration );

        // total interrupt
        long lOldPause = _clnOldPause.getTime().getTime();
        Calendar clnTotal = (Calendar)_clnOldPause.clone();
        clnTotal.setTime( new Date( lOldPause + lNow - lStart ) );
        String sTotal = Util.getTime( clnTotal );

        _txtDuration.setText( sDuration );
        _txtTotal.setText( sTotal );
    }

    /**
     * @param sPause_ The total time older pauses have taken
     *                so far.
     */
    public PauseDialog(Controllable pControllable_, String sPause_) {
        super( (JFrame)pControllable_, S_TITLE, true );

        Util.debug( this, "<init>(..).sPause_: " + sPause_ );
        Util.panicIf(sPause_ == null);

        Init pInit = pControllable_.getInit();
        String sTimezoneID = pInit.getKeyValue( "TimezoneID" );
        if ( !Util.isEmpty( sTimezoneID ) ) {
            sTimezoneID = "GMT";
        }

        _bOk = false;
        setBackground(((JFrame)pControllable_).getBackground());
        setForeground(((JFrame)pControllable_).getForeground());
        
        // the time the pause starts
        _clnStart = Util.getCalendar( sTimezoneID );

        // duration / this pause
        _clnNull = Util.getCalendar( sTimezoneID );
        Util.setTime( _clnNull, "00:00:00" );
        _clnNull.setTime( new Date( _clnNull.getTime().getTime() / 1000 * 1000 ) );
        
        // total pause
        _clnOldPause = Util.getCalendar( sTimezoneID );
        Util.setTime( _clnOldPause, sPause_ );

        getContentPane().setLayout(new GridLayout(3, 3, 1, 1));
        getContentPane().add(new JLabel("Start"));
        getContentPane().add(new JLabel("This Interrupt"));
        getContentPane().add(new JLabel("Total Interrupt"));

        _txtStart.setText( Util.getTime( _clnStart ) );
        _showTime();

        _txtStart.setEditable(false);
        _txtDuration.setEditable(false);
        _txtTotal.setEditable(false);
        getContentPane().add(_txtStart);
        getContentPane().add(_txtDuration);
        getContentPane().add(_txtTotal);
        getContentPane().add(_btnOk);
        _btnOk.addActionListener(this);
        getContentPane().add(new JLabel(" "));
        getContentPane().add(_btnCancel);
        _btnCancel.addActionListener(this);
        
        _pThread = new Thread(this);
        _pThread.start();
        
        pack();
        SwingUtil.centerComponent(this);
    }

    public String getNewPause() {
        Calendar clnNewPause = (Calendar)_clnNull.clone();

        long lTotal = _clnOldPause.getTime().getTime();
        if (_bOk) {
            lTotal += (new Date()).getTime() - _clnStart.getTime().getTime();
        }
        clnNewPause.setTime( new Date( lTotal ) );

        return Util.getTime( clnNewPause );
    }

    boolean _bExit = false;

    public void run() {
        while( !_bExit ) {
            try {
                _pThread.sleep(1000);
            } catch(Exception e) {
            }
            Util.debug("PauseDialog: run");
            _showTime();
        }
        setVisible( false );
    }

    public void exit() {
        _bExit = true;
        _pThread.interrupt();
    }

    public void actionPerformed(ActionEvent pActionEvent_) {
        Object oSource = pActionEvent_.getSource();
        if (oSource == _btnOk) {
            _bOk = true;
            exit();
        }
        if (oSource == _btnCancel) {
            exit();
        }
    }
}
