/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package psp.timelog;

import java.awt.Frame;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JLabel;
import javax.swing.JPanel;

import ccl.swing.AutoGridBagLayout;
import ccl.swing.DateField;
import ccl.swing.ExitJDialog;
import ccl.swing.OKCancelPanel;
import ccl.swing.SwingUtil;
import ccl.util.Init;

/**
 * @version $Id: PeriodeDialog.java,v 1.2 2003/05/03 12:13:00 c1emens Exp $
 * @author  Chr. Clemens Lee
 */
public class PeriodeDialog extends ExitJDialog {
    private DateField _dtfStart = null;
    private DateField _dtfEnd   = null;

    private OKCancelPanel _pOKCancelPanel = null;

    public PeriodeDialog( Controllable controllable_ ) {
        super( (Frame)controllable_, "Timelog: Export Periode Dialog" );

        Init init = controllable_.getInit();
        String sTimezoneID = init.getKeyValue( "TimezoneID" );

        _dtfStart = new DateField( sTimezoneID );
        _dtfEnd   = new DateField( sTimezoneID );

        _pOKCancelPanel = new OKCancelPanel( this );

        JPanel pnlContent = (JPanel)getContentPane();
        pnlContent.setBorder( BorderFactory.createEmptyBorder
                              ( 5, 5, 5, 5 ) );
 
        AutoGridBagLayout layout = null;

        layout = new AutoGridBagLayout();
        layout.setLayoutOn( pnlContent );

        layout.setAnchor( AutoGridBagLayout.WEST );
        layout.setExtend( 5, 1 );
        layout.add( pnlContent, new JLabel( "Please enter start and end dates for periode" ) );
        layout.endLine();
        
        layout.add( pnlContent, new JLabel( "(excluding end date) to export (leave fields" ) );
        layout.endLine();
        layout.add( pnlContent, new JLabel( "blank to export entire data set)." ) );
        layout.endLine();

        layout.setExtend( 1, 1 );
        layout.add( pnlContent, new JLabel( "Start Date: " ) );
        layout.setFillHorizontal();
        layout.add( pnlContent, _dtfStart );
        layout.setFillNone();

        layout.setExpandHorizontal();
        layout.add( pnlContent, new JLabel( " " ) );
        layout.setExpandNone();

        layout.add( pnlContent, new JLabel( "End Date: " ) );
        layout.setFillHorizontal();
        layout.add( pnlContent, _dtfEnd );
        layout.setFillNone();
        layout.endLine();

        layout.add( pnlContent, Box.createVerticalStrut
                    ( 5 ) );
        layout.endLine();

        layout.setExtend( 5, 1 );
        layout.setAnchor( AutoGridBagLayout.EAST );
        layout.add( pnlContent, _pOKCancelPanel );
        layout.setExtend( 1, 1 );

        pack();
        setSize( getPreferredSize() );

        SwingUtil.centerComponent( this );
    }

    public String getStartDate() {
        return _dtfStart.getText();
    }

    public String getEndDate() {
        return _dtfEnd.getText();
    }

    public boolean isOK() {
        return _pOKCancelPanel.isOK();
    }
}
