/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package psp.timelog;

import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import psp.timelog.util.ObservableInt;
import psp.timelog.util.ObservableShortDate;
import psp.timelog.util.ObservableString;
import psp.timelog.util.ObservableTime;
import psp.timelog.util.ObservableValue;
import ccl.util.Init;
import ccl.util.Util;

/**
 * @author <a href="http://www.kclee.com/clemens/">
 *         Chr. Clemens Lee</a>
 *         (<a href="mailto:clemens@kclee.com">
 *         clemens@kclee.com
 *         </a>)
 *         , Kenneth Westelinck (kenneth.westelinck@pandora.be)
 * @version $Id: Session.java,v 1.2 2003/05/03 12:13:00 c1emens Exp $
 */
public class Session implements Observer {
    public static final int DATE = 0;
    public static final int START = 1;
    public static final int STOP = 2;
    public static final int PAUSE = 3;
    public static final int DELTA = 4;
    public static final int PHASE = 5;
    public static final int COMMENT = 6;
    public static final int FIELDS = 7;

    private ObservableShortDate _obldDate;
    private ObservableTime      _obltStart;
    private ObservableTime      _obltStop;
    private ObservableTime      _obltPause;
    private ObservableTime      _obltDelta;
    private ObservableInt       _obliPhase;
    private ObservableString    _oblsComment;
    private Init _pInit;

    private void _setDateNow() {
        String sToday = Util.getDate
               ( Util.getCalendar
                 ( _pInit.getKeyValue
                   ( Controller.S_INIT_TIMEZONE_ID ) ) );
        _obldDate.set(sToday);
    }

    private String _getNow() {
        String sNow = Util.getTime
               ( Util.getCalendar
                 ( _pInit.getKeyValue
                   ( Controller.S_INIT_TIMEZONE_ID ) ) );

        return sNow;
    }

    private void _setStartNow() {
        String sNow = _getNow();

        _obltStart.set(sNow);
    }

    public Session(Init pInit_) {
        super();
        _pInit = pInit_;
        _obldDate = new ObservableShortDate();
        _obltStart = new ObservableTime("00:00:00");
        _obltStop = new ObservableTime("00:00:00");
        _obltPause = new ObservableTime("00:00:00");
        _obltDelta = new ObservableTime("00:00:00");
        _obliPhase = new ObservableInt(0);
        _oblsComment = new ObservableString();
    }
    
    public void setDateAndStartNow() {
        _setDateNow();
        _setStartNow();
        _obltPause.set("00:00:00");
    }
    
    public void setStopNow() {
        String sNow = _getNow();
        
        _obltStop.set(sNow);
    }
    
    public static String getTimeFromSeconds( int seconds_ ) {
        Util.panicIf(seconds_ < 0 || (24*60*60) <= seconds_);
        int hours = seconds_ / 3600;
        int minutes = (seconds_ - hours*3600) / 60;
        int secs = seconds_ - hours*3600 - minutes*60;
        
        String sRetVal = Util.paddWithZero(hours, 2) + ":" +
               Util.paddWithZero(minutes, 2) + ":" +
               Util.paddWithZero(secs, 2);
        
        return sRetVal;
    }

    public void update(Observable pObservable_, Object pObject_) {
        // Delta muss neu bestimmt werden
        // wird aufgerufen, wenn sich start, stop oder pause ndert
        int secondsStart = Timelog.timeToSeconds(_obltStart.
                                                 toString());
        int secondsStop = Timelog.timeToSeconds(_obltStop.
                                                toString());
        if (secondsStop < secondsStart) {
            secondsStop += (24*60*60);
        }
        int secondsPause = Timelog.timeToSeconds(_obltPause.
                                                 toString());
        int secondsDelta = secondsStop - secondsStart - secondsPause;
        if (secondsDelta < 0) {
            secondsDelta += (24*60*60);
        }
        secondsDelta = secondsDelta % (24*60*60);
        String sDeltaTime = getTimeFromSeconds( secondsDelta );
        _obltDelta.set(sDeltaTime);
        Util.debug("Session: update: sDeltaTime, secondsDelta: " +
                   sDeltaTime + ", " + secondsDelta);
    }
    
	public void set(String[] asSessionValue_) {
		Util.panicIf(asSessionValue_.length != FIELDS);

		_obldDate.set(asSessionValue_[DATE]);
		_obltStart.set(asSessionValue_[START]);
		_obltStop.set(asSessionValue_[STOP]);
		_obltPause.set(asSessionValue_[PAUSE]);
		_obltDelta.set(asSessionValue_[DELTA]);
		_obliPhase.set(asSessionValue_[PHASE]);
		_oblsComment.set(asSessionValue_[COMMENT]);
	}

    public void set(Vector vSession_) {
        Util.panicIf(vSession_.size() != FIELDS);
        
        Util.debug("Session: set: vSession_[DATE]: " +
                   (String)vSession_.elementAt(DATE));
        _obldDate.set((String)vSession_.elementAt(DATE));
        _obltStart.set((String)vSession_.elementAt(START));
        _obltStop.set((String)vSession_.elementAt(STOP));
        _obltPause.set((String)vSession_.elementAt(PAUSE));
        Util.debug("Session: set: vSession_[PAUSE]: " +
                   (String)vSession_.elementAt(PAUSE));
        _obltDelta.set((String)vSession_.elementAt(DELTA));
        _obliPhase.set((String)vSession_.elementAt(PHASE));
        _oblsComment.set((String)vSession_.elementAt(COMMENT));
    }

    public ObservableValue getDate() {
        return((ObservableValue)_obldDate);
    }

	public ObservableValue getStart() {
		return((ObservableValue)_obltStart);
	}

	public ObservableValue getStop() {
		return((ObservableValue)_obltStop);
	}

	public ObservableValue getPause() {
		return((ObservableValue)_obltPause);
	}

	public ObservableValue getDelta() {
		return((ObservableValue)_obltDelta);
	}

	public ObservableValue getPhase() {
		return((ObservableValue)_obliPhase);
	}
    
    public ObservableValue getComment() {
        return((ObservableValue)_oblsComment);
    }
    
    public String toString() {
        String sRetVal = ((ObservableShortDate)getDate()).getDate()
               + "\t" + getStart().toString() + "\t" +
               getStop().toString() + "\t" + getPause().toString() +
               "\t" + getDelta().toString() + "\t" + getPhase() + "\t" +
               Util.replace(getComment().toString(), "\t", "   ");
        
        return sRetVal;
    }
}
