/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package psp.timelog;

import ccl.swing.AutoGridBagLayout;
import ccl.util.Util;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Calendar;
import java.util.Date;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import psp.timelog.util.ObservableValue;

/**
 * @author <a href="http://www.kclee.com/clemens/">
 *         Chr. Clemens Lee</a>
 *         (<a href="mailto:clemens@kclee.com">
 *         clemens@kclee.com
 *         </a>)
 * @version $Id: SessionView.java,v 1.3 2003/05/03 12:13:00 c1emens Exp $
 */
public class SessionView extends JPanel implements ActionListener {
    public static final String S_LAST_COMMENT = " Last Comment ";
    public static final String S_CONTINUE = " Continue ";
    private JButton _btnTodayAndNow;
    private JButton _btnContinue;
    private JButton _btnNow;
    private JButton _btnPause;
    private JButton _btnComment;
    private TextFieldObserver _txtDate;
    private TextFieldObserver _txtStart;
    private TextFieldObserver _txtStop;
    private TextFieldObserver _txtInterrupt;
    private TextFieldObserver _txtDelta;
    private ChoiceObserver    _chcPhase;
    private TextFieldObserver _txtComment;

    private Session _pSession;
    private Controller _pController;
    private Timelog _pTimelog;

    public void pause() {
        Util.debug("SessionView.pause().START");
        String sPause = _txtInterrupt.getText();
        _pSession.getPause().set(sPause);
        PauseDialog pPauseDialog =
            new PauseDialog(_pController, _pSession.getPause().toString());
        pPauseDialog.show();
        String sNewPause = pPauseDialog.getNewPause();
        _pSession.getPause().set(sNewPause);
        pPauseDialog.dispose();
        _pController.requestFocus();
        Util.debug("SessionView.pause().END");
    }

    /**
     * Set start date and time to the stop values of the
     * selected row.
     */
    private void _setStartContinue() {
        // date is in format 12/31/99
        String sDate = _pTimelog.getLastDate();
        String sStart = _pTimelog.getLastStart();
        String sStop = _pTimelog.getLastStop();

        Calendar clnStart = Util.getDate( Timelog.normalizeDate( sDate ) );
        Calendar clnStop = Util.getDate( Timelog.normalizeDate( sDate ) );

        Util.setTime( clnStart, sStart );
        Util.setTime( clnStop, sStop );

        if ( clnStop.getTime().getTime() 
             < clnStart.getTime().getTime() )
        {
            // + 24h
            clnStop.setTime( new Date( clnStop.getTime()
                                              .getTime() 
                                       + 1000 * 60 * 60 * 24 ) );
            sDate = Util.getDate( clnStop );
            sDate =
                sDate.substring(5, 7)
                    + "/"
                    + sDate.substring(8)
                    + "/"
                    + sDate.substring(2, 4);
        }

        ObservableValue oblvDate = _pSession.getDate();
        oblvDate.set(sDate);
        ObservableValue oblvStart = _pSession.getStart();
        oblvStart.set(sStop);

        ObservableValue oblvPause = _pSession.getPause();
        oblvPause.set("00:00:00");
    }

    public SessionView( Session pSession_
                        , Controller pController_
                        , Timelog pTimelog )
    {
        super();

        _pSession = pSession_;
        _pController = pController_;
        _pTimelog = pTimelog;

        Insets buttonMargin = Controller.getButtonMargin();

        _btnTodayAndNow = new JButton("Now");
        _btnContinue = new JButton(S_CONTINUE);
        _btnNow = new JButton("Now");
        _btnPause = new JButton("Pause");
        _btnComment = new JButton(S_LAST_COMMENT);

        _chcPhase =
            new ChoiceObserver(
                _pSession.getPhase(),
                _pController.getProcessElements());

        Controller.setFontOn( _btnTodayAndNow );
        Controller.setFontOn( _btnContinue );
        Controller.setFontOn( _btnNow );
        Controller.setFontOn( _btnPause );
        Controller.setFontOn( _btnComment );
        
        Controller.setFontOn( _chcPhase );

        _pController.setBorderOn( _btnTodayAndNow );
        _pController.setBorderOn( _btnContinue );
        _pController.setBorderOn( _btnNow );
        _pController.setBorderOn( _btnPause );
        _pController.setBorderOn( _btnComment );
        
        _pController.setBorderOn( _chcPhase );

        _btnTodayAndNow.setMargin( buttonMargin );
        _btnContinue.setMargin( buttonMargin );
        _btnNow.setMargin( buttonMargin );
        _btnPause.setMargin( buttonMargin );
        _btnComment.setMargin( buttonMargin );

        _btnContinue.setMinimumSize( new Dimension( getPreferredButtonSize().width
                                                    , getPreferredButtonSize().height ) );//- 6 ) );
        _btnTodayAndNow.setMinimumSize( getPreferredButtonSize() );
        _btnNow.setMinimumSize( getPreferredButtonSize() );
        _btnPause.setMinimumSize( getPreferredButtonSize() );
        _btnComment.setMinimumSize( new Dimension( _btnComment.getMinimumSize().width
                                                   , getPreferredButtonSize().height ) );
        _chcPhase.setMinimumSize( new Dimension( 0 //_chcPhase.getMinimumSize().width
                                                 , getPreferredButtonSize().height ) );

        AutoGridBagLayout pAutoGridBagLayout = new AutoGridBagLayout();
        setLayout(pAutoGridBagLayout);
        pAutoGridBagLayout.setInsets( 2 );

        JLabel lblTemp = new JLabel("Date");
        Controller.setFontOn( lblTemp );
        add(lblTemp);

        lblTemp = new JLabel("Start");
        Controller.setFontOn( lblTemp );
        add( lblTemp );

        pAutoGridBagLayout.setExtend(2, 1);

        lblTemp = new JLabel("Stop");
        Controller.setFontOn( lblTemp );
        add( lblTemp );

        pAutoGridBagLayout.setExtend(1, 1);

        lblTemp = new JLabel("Interrupt");
        Controller.setFontOn( lblTemp );
        add( lblTemp );

        pAutoGridBagLayout.endLine();

        _btnTodayAndNow.addActionListener(this);
        pAutoGridBagLayout.setFillBoth();
        add(_btnTodayAndNow);
        pAutoGridBagLayout.setFillNone();

        _btnContinue.addActionListener(this);
        add(_btnContinue);

        _btnNow.addActionListener(this);
        pAutoGridBagLayout.setExtend(2, 1);
        add(_btnNow);
        pAutoGridBagLayout.setExtend(1, 1);

        _btnPause.addActionListener(_pController);
        add(_btnPause);
        pAutoGridBagLayout.endLine();

        _txtDate = new TextFieldObserver(_pSession.getDate(), 7);
        _txtDate.setMinimumSize( getPreferredButtonSize() );
        _pSession.getDate().addObserver(_txtDate);
        pAutoGridBagLayout.setFillHorizontal();
        add(_txtDate);

        _txtStart = new TextFieldObserver(_pSession.getStart(), 7);
        _txtStart.setMinimumSize( getPreferredButtonSize() );
        _pSession.getStart().addObserver(_txtStart);
        _pSession.getStart().addObserver(_pSession);
        add(_txtStart);

        _txtStop = new TextFieldObserver(_pSession.getStop(), 7);
        _txtStop.setMinimumSize( getPreferredButtonSize() );
        _pSession.getStop().addObserver(_txtStop);
        _pSession.getStop().addObserver(_pSession);
        pAutoGridBagLayout.setExtend(2, 1);
        add(_txtStop);
        pAutoGridBagLayout.setExtend(1, 1);

        _txtInterrupt = new TextFieldObserver(_pSession.getPause(), 7);
        _txtInterrupt.setMinimumSize( getPreferredButtonSize() );
        _pSession.getPause().addObserver(_txtInterrupt);
        _pSession.getPause().addObserver(_pSession);
        add(_txtInterrupt);
        pAutoGridBagLayout.endLine();

        // empty line
        add( Controller.createVerticalStrut());
        pAutoGridBagLayout.endLine();

        pAutoGridBagLayout.setAnchor( AutoGridBagLayout.CENTER );
        pAutoGridBagLayout.setFillNone();
        lblTemp = new JLabel("Delta");
        Controller.setFontOn( lblTemp );
        add( lblTemp );
        lblTemp = new JLabel("Phase");
        Controller.setFontOn( lblTemp );
        add( lblTemp );
        add(new JLabel(" "));
        add(new JLabel(" "));
        //pAutoGridBagLayout.setExtend(2, 1);
        lblTemp = new JLabel("Comment");
        Controller.setFontOn( lblTemp );
        add( lblTemp );
        pAutoGridBagLayout.setExtend(1, 1);
        pAutoGridBagLayout.endLine();

        _txtDelta = new TextFieldObserver(_pSession.getDelta(), 7);
        _txtDelta.setEnabled(false);
	_txtDelta.setDisabledTextColor( _txtStart.getForeground() );
	_txtDelta.setBackground( getBackground() );
        _txtDelta.setMinimumSize( getPreferredButtonSize() );
        _pSession.getDelta().addObserver(_txtDelta);
        pAutoGridBagLayout.setFillHorizontal();
        add(_txtDelta);

        _pSession.getPhase().addObserver(_chcPhase);
        pAutoGridBagLayout.setExtend(2, 1);
        pAutoGridBagLayout.setAnchor(AutoGridBagLayout.WEST);
        add(_chcPhase);
        pAutoGridBagLayout.setAnchor(AutoGridBagLayout.CENTER);
        pAutoGridBagLayout.setExtend(1, 1);

        _btnComment.addActionListener(_pController);
        pAutoGridBagLayout.setExtend(2, 1);
        pAutoGridBagLayout.setAnchor(AutoGridBagLayout.EAST);
        pAutoGridBagLayout.setFillNone();
        add(_btnComment);
        pAutoGridBagLayout.setAnchor(AutoGridBagLayout.CENTER);
        pAutoGridBagLayout.setExtend(1, 1);
        pAutoGridBagLayout.endLine();

        _txtComment = new TextFieldObserver(_pSession.getComment());
        _txtComment.setMinimumSize( getPreferredButtonSize() );
        _pSession.getComment().addObserver(_txtComment);

        pAutoGridBagLayout.setAnchor(AutoGridBagLayout.EAST);
        pAutoGridBagLayout.setExtend(5, 1);
        pAutoGridBagLayout.setFillHorizontal();
        add(_txtComment);
        pAutoGridBagLayout.setFillNone();
        pAutoGridBagLayout.setExtend(1, 1);
        pAutoGridBagLayout.setAnchor(AutoGridBagLayout.CENTER);
    }

    /**
     * Returns the preferred button size, as e.g. specified by
     * the widest button, the continue button.
     * If the continue button is not set up yet, this method
     * does not return anything valuable.
     */
    public Dimension getPreferredButtonSize()
    {
        return _btnContinue.getMinimumSize();
    }

    public void updatePhaseChoice() {
        _chcPhase.updateItems(_pController.getProcessElements());
    }

    public void updateSession() {
        _pSession.getDate().set(_txtDate.getText());
        _pSession.getStart().set(_txtStart.getText());
        _pSession.getStop().set(_txtStop.getText());
        _pSession.getPause().set(_txtInterrupt.getText());
        _pSession.getComment().set(_txtComment.getText());
    }

    public void actionPerformed(ActionEvent pActionEvent_) {
        Object oSource = pActionEvent_.getSource();
        if (oSource == _btnNow) {
            _pSession.setStopNow();
        } else if (oSource == _btnTodayAndNow) {
            _pSession.setDateAndStartNow();
        } else if (oSource == _btnContinue) {
            _setStartContinue();
        }
    }
}
