/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package psp.timelog;

import ccl.util.Util;
import java.awt.BorderLayout;
import java.util.Enumeration;
import java.util.Observable;
import java.util.Observer;
import javax.swing.JPanel;
import psp.timelog.table.SummaryModel;
import psp.timelog.table.TimelogTable;

/**
 * @author Chr. Clemens Lee (clemens@kclee.com)
 *         , Kenneth Westelinck (kenneth.westelinck@pandora.be)
 * @version $Id: SummaryView.java,v 1.2 2003/05/03 12:13:00 c1emens Exp $
 */
public class SummaryView extends    JPanel
                         implements Observer
{
    private static String[] asTitle =
    {
        "Phase", "Total time", "Last week", "Last day"
    };

    private Controller _pController;
    private Timelog _pTimelog;
    private SummaryModel summaryModel = null;

    public SummaryView(Controller pController_, Timelog pTimelog_)
    {
        super();

        _pController = pController_;
        _pTimelog = pTimelog_;
        _pTimelog.addObserver(this);
        init();
    }

    private void init()
    {
        setLayout(new BorderLayout());

        summaryModel = new SummaryModel(asTitle);
        summaryModel.setData(createSummary());

        TimelogTable scrollTable = new TimelogTable( summaryModel, Controller.getStandardFont() );
        add( scrollTable );
    }

    public static String formatTime( long lSeconds_ )
    {
        if ( lSeconds_ <= 0 )
        {
            return "";
        }

        long lMinutes = lSeconds_ / 60;
        long lHours = lMinutes / 60;
        lMinutes = lMinutes - lHours * 60;

        String sRetVal =
               ""
               + Util.paddWithZero((int) lHours, 2)
               + " h "
               + Util.paddWithZero((int) lMinutes, 2)
               + " m";

        return sRetVal;
    }

    /**
     * In Timelog summary data is store in a long table,
     * while this method returns a string table ready to
     * print out.
     */
    public String[][] createSummary()
    {
        long[][] aalSummary = _pTimelog.getSummary();
        int processSize = _pController.getProcessSize();
        String[][] aasSummary = new String[processSize + 1][Timelog.SUM_FIELDS + 1];
        long[] alTotal = new long[Timelog.SUM_FIELDS];

        int phase = 0;
        for ( Enumeration e = _pController.getProcessElements()
                     ; e.hasMoreElements()
                     ; )
        {
            aasSummary[phase][0] = (String) e.nextElement();
            long lSeconds = aalSummary[phase][Timelog.SUM_TOTAL];
            alTotal[Timelog.SUM_TOTAL] += lSeconds;
            aasSummary[phase][Timelog.SUM_TOTAL + 1] = formatTime(lSeconds);
            aasSummary[phase][2] = "";
            lSeconds = aalSummary[phase][Timelog.SUM_WEEK];
            alTotal[Timelog.SUM_WEEK] += lSeconds;
            aasSummary[phase][Timelog.SUM_WEEK + 1] = formatTime(lSeconds);
            lSeconds = aalSummary[phase][Timelog.SUM_DAY];
            alTotal[Timelog.SUM_DAY] += lSeconds;
            aasSummary[phase][Timelog.SUM_DAY + 1] = formatTime(lSeconds);
            phase++;
        }

        aasSummary[processSize][0] = "Total";
        aasSummary[processSize][Timelog.SUM_TOTAL + 1] = formatTime(alTotal[Timelog.SUM_TOTAL]);
        aasSummary[processSize][Timelog.SUM_WEEK  + 1] = formatTime(alTotal[Timelog.SUM_WEEK]);
        aasSummary[processSize][Timelog.SUM_DAY   + 1] = formatTime(alTotal[Timelog.SUM_DAY]);

        return aasSummary;
    }

    public void update(Observable pObservable_, Object pObject_)
    {
        Util.debug("SummaryView: update: ");
        String[][] aasSummary = createSummary();
        summaryModel.clear();
        summaryModel.setData(aasSummary);
        summaryModel.update();
    }
}
