/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package psp.timelog;

import ccl.swing.SwingUtil;
import ccl.util.Util;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.Observable;
import java.util.Observer;
import javax.swing.JTextField;
import psp.timelog.util.ObservableValue;

/**
 * @author <a href="http://www.kclee.com/clemens/">
 *         Chr. Clemens Lee</a>
 *         (<a href="mailto:clemens@kclee.com">
 *         clemens@kclee.com
 *         </a>)
 */
public class TextFieldObserver extends JTextField
                               implements Observer
					  , KeyListener 
					  , FocusListener
{
    private ObservableValue _oblvText = null;

    public TextFieldObserver(ObservableValue oblvText_) {
        super(oblvText_.toString());
        
        setBorder( SwingUtil.createCCLBorder() );
        addKeyListener( this );
	addFocusListener( this );
        _oblvText = oblvText_;
        setText(_oblvText.toString());
    }

    public TextFieldObserver(ObservableValue oblvText_, int columns_) {
        super(oblvText_.toString(), columns_);

        setBorder( SwingUtil.createCCLBorder() );
        addKeyListener(this);
	addFocusListener( this );
        _oblvText = oblvText_;
        setText(_oblvText.toString());
    }

    public void update(Observable pObservable_, Object pObject_) {
        Util.debug("TextFieldObserver: update: " + this);
        Util.debug("TextFieldObserver: pObservable_: " + pObservable_);
        setText(pObservable_.toString());
    }
    
    /**
     * Updates text field when user moves on.
     */
    public void keyPressed( KeyEvent pKeyEvent_ )
    {
        Util.debug( "TextFieldObserver: keyPressed: pKeyEvent_: "
                    + pKeyEvent_
                    + ", Enter: "
                    + KeyEvent.VK_ENTER );

        // tab or enter
	// with jdk 1.3 focus changes we don't get tab here anymore, use focus lost instead
        if ( pKeyEvent_.getKeyCode() == KeyEvent.VK_ENTER )
	{
            /*|| (pKeyEvent_.getKeyCode() == KeyEvent.VK_TAB)) {*/
            Util.debug("TextFieldObserver: keyPressed: ENTER: ");
            _oblvText.set(getText());
        }
    }

    public void keyReleased(KeyEvent pKeyEvent_) {
    }

    public void keyTyped(KeyEvent pKeyEvent_) {
    }

    /**
     * Updates text field when user moves on.
     */
    public void focusLost( FocusEvent event ) 
    {
	_oblvText.set(getText());
    }

    /**
     * Empty implementation of the focus listener interface.
     */
    public void focusGained( FocusEvent event )
    {
    }
}
