/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package psp.timelog;

import java.util.Calendar;
import java.util.Date;
import java.util.Enumeration;
import java.util.Observable;
import java.util.Vector;
import javax.swing.JFrame;

import ccl.swing.SwingUtil;
import ccl.util.FileUtil;
import ccl.util.Init;
import ccl.util.Util;

/**
 * @author <a href="http://www.kclee.com/clemens/">
 *         Chr. Clemens Lee</a>
 *         (<a href="mailto:clemens@kclee.com">
 *         clemens@kclee.com
 *         </a>)
 *         , Kenneth Westelinck (kenneth.westelinck@pandora.be)
 */
public class Timelog extends Observable
{
    public static final int NOTHING_SELECTED = -1;
    private Vector _vSessions = new Vector();
    private Controllable _pControllable;
    private Session _pSession;
    private int _selectedSession = NOTHING_SELECTED;
    private long _aalSummary[][];
    public static final int SUM_TOTAL = 0;
    public static final int SUM_WEEK = 1;
    public static final int SUM_DAY = 2;
    public static final int SUM_FIELDS = 3;

    /** Used for user convenience in this.export() */
    private String _sLastDirectory = null;

    /** Used for user convenience in this.export() */
    private String _sLastExportFile = null;

    private void _clear() {
        _vSessions = new Vector();
	_selectedSession = NOTHING_SELECTED;
        // zu diesem Zeitpunkt muss der Process feststehen
        _aalSummary = new long[_pControllable.getProcessSize()][SUM_FIELDS];
        _pControllable.setModified();
        setChanged();
        notifyObservers();
        _sLastDirectory = null;
        _sLastExportFile = null;
    }

    public String[][] getSessions( boolean bPhaseAsString_ )
    {
        if (_vSessions.size() == 0) {
            return (String[][]) null;
        }
        String[][] aasSession = new String[_vSessions.size()][Session.FIELDS];
        int session = 0;
        int element = 0;
        for ( Enumeration eSessions = _vSessions.elements();
              eSessions.hasMoreElements(); ) 
        {
            for (Enumeration eElements =
                        ((Vector) eSessions.nextElement()).elements();
                 eElements.hasMoreElements(); )
            {
                String sNextElement = (String) eElements.nextElement();
                if (element == 0) {
                    sNextElement = normalizeDate(sNextElement);
                }
                aasSession[session][element] = sNextElement;
                element++;
            }

            // Phase als Text
            if (bPhaseAsString_) {
                aasSession[session][Session.PHASE] =
                    _pControllable.getProcessElement(
                        aasSession[session][Session.PHASE]);
                if (aasSession[session][Session.PAUSE].equals("00:00:00")) {
                    aasSession[session][Session.PAUSE] = "";
                }
            }

            element = 0;
            session++;
        }
        //Util.debug("Timelog: _vSessions: " + _vSessions);

        return aasSession;
    }

    private void _clearSummary(int index_) {
        if (_aalSummary != null) {
            for (int phase = 0; phase < _aalSummary.length; phase++) {
                _aalSummary[phase][index_] = 0;
            }
        }
    }

    /**
     * @param   sTime_   e.g. "23:59:59".
     */
    public static int timeToSeconds(String sTime_) {
        int retVal =
            Util.atoi( sTime_.substring( 6 ) )
                + Util.atoi( sTime_.substring( 3, 5 ) ) * 60
                + Util.atoi( sTime_.substring( 0, 2 ) ) * 60 * 60;

        return retVal;
    }

    private void _setDaily() {
        _clearSummary(SUM_DAY);
        if (_vSessions.size() == 0) {
            return;
        }
        String sLastDate =
            (String) ((Vector) _vSessions.lastElement()).elementAt(
                Session.DATE);
        int indexSession = _vSessions.size() - 1;
        Vector vSession = (Vector) _vSessions.elementAt(indexSession);
        String sDate = "";
        do {
            String sPhase = (String) vSession.elementAt(Session.PHASE);
            String sDelta = (String) vSession.elementAt(Session.DELTA);

            int phase = Util.atoi(sPhase);
            Util.debug("Timelog: _setDaily: phase: " + phase);
            _aalSummary[phase][SUM_DAY] += timeToSeconds(sDelta);
            indexSession--;
            if (indexSession >= 0) {
                vSession = (Vector) _vSessions.elementAt(indexSession);
                sDate = (String) vSession.elementAt(Session.DATE);
            } else {
                break;
            }
        } while (sDate.equals(sLastDate));
    }

    private long _getDays( String sDate_ )
    {
        Calendar pCalendar = Util.getDate( normalizeDate( sDate_ ) );

        long lMillis = pCalendar.getTime().getTime();
        long lDays = (((lMillis / 1000) / 60) / 60) / 24;

        return lDays;
    }

    private void _setWeekly() {
        _clearSummary(SUM_WEEK);
        if (_vSessions.size() == 0) {
            return;
        }
        String sLastDate = (String) ((Vector) _vSessions.lastElement())
               .elementAt( Session.DATE );
        long lLastDay = _getDays( sLastDate );
        int indexSession = _vSessions.size() - 1;
        Vector vSession = (Vector) _vSessions.elementAt(indexSession);
        String sDate = "";
        long lDay = 0;
        do
        {
            String sPhase = (String) vSession.elementAt(Session.PHASE);
            String sDelta = (String) vSession.elementAt(Session.DELTA);
            int phase = Util.atoi(sPhase);
            Util.debug("Timelog: _setDaily: phase: " + phase);
            _aalSummary[phase][SUM_WEEK] += timeToSeconds(sDelta);
            indexSession--;
            if (indexSession >= 0) {
                vSession = (Vector) _vSessions.elementAt(indexSession);
                sDate = (String) vSession.elementAt(Session.DATE);
                lDay = _getDays( sDate );
            }
            else
            {
                break;
            }
        } while ( lDay > (lLastDay - 7) );
    }

    private void _addSession(String sSession_, boolean bUpdate_) {
        Vector vSession = Util.stringToLines(sSession_, '\t');
        if (vSession.size() == Session.FIELDS - 1) {
            vSession.addElement("");
        }
        Util.panicIf(
            vSession.size() != Session.FIELDS,
            "Timelog: addSession: vSession: " + vSession);
        _vSessions.addElement(vSession);
        // Total Time Count
        int phase = Util.atoi((String) vSession.elementAt(Session.PHASE));
        String sDelta = (String) vSession.elementAt(Session.DELTA);
        _aalSummary[phase][SUM_TOTAL] += timeToSeconds(sDelta);

        _selectedSession = _vSessions.size() - 1;
        if (bUpdate_) {
            setChanged();
            notifyObservers();
        }
    }

    public Timelog( Controllable pControllable_
                    , Session pSession_ )
    {
        super();

        _pControllable = pControllable_;
        _pSession = pSession_;
    }

    public void addSession(String sSession_) {
        _pControllable.setModified();
        _addSession(sSession_, true);
    }

    public void deleteSession() {
        if (_selectedSession == NOTHING_SELECTED) {
            SwingUtil.showMessage( (JFrame) _pControllable
                                   , "There is no selected Session.\nPlease select a Session in the List Box before you press the delete button.");

            return;
        }

        _pControllable.setModified();

        // Total Time Count
        Vector vSession = (Vector) _vSessions.elementAt(_selectedSession);
        int phase = Util.atoi((String) vSession.elementAt(Session.PHASE));
        String sDelta = (String) vSession.elementAt(Session.DELTA);
        _aalSummary[phase][SUM_TOTAL] -= timeToSeconds(sDelta);

        _vSessions.removeElementAt(_selectedSession);
        if (_selectedSession >= _vSessions.size()) {
            _selectedSession = _vSessions.size() - 1;
            if (_selectedSession < 0) {
                _selectedSession = NOTHING_SELECTED;
            }
        }

        setChanged();
        notifyObservers();
    }

    public void changeSession() {
        if ( _selectedSession == NOTHING_SELECTED )
        {
            SwingUtil.showMessage( (JFrame) _pControllable
                                   , "There is no selected Session.\nPlease select a Session in the List Box before you press the change button." );

            return;
        }

        _pControllable.setModified();

        Vector vSession = Util.stringToLines(_pSession.toString(), '\t');
        if (vSession.size() == Session.FIELDS - 1) {
            vSession.addElement("");
        }
        Util.panicIf(
            vSession.size() != Session.FIELDS,
            "Timelog: changeSession: vSession: " + vSession);

        // Total Time Count Old
        Vector vOldSession = (Vector) _vSessions.elementAt(_selectedSession);
        int oldPhase = Util.atoi((String) vOldSession.elementAt(Session.PHASE));
        String sOldDelta = (String) vOldSession.elementAt(Session.DELTA);
        _aalSummary[oldPhase][SUM_TOTAL] -= timeToSeconds(sOldDelta);

        _vSessions.setElementAt(vSession, _selectedSession);

        // Total Time Count New
        int phase = Util.atoi((String) vSession.elementAt(Session.PHASE));
        String sDelta = (String) vSession.elementAt(Session.DELTA);
        _aalSummary[phase][SUM_TOTAL] += timeToSeconds(sDelta);

        setChanged();
        notifyObservers();
    }

    private static boolean _isGermanDate( String sDate_ )
    {
        if ( sDate_.length() == 12
             && sDate_.charAt(2) == '.'
             && sDate_.charAt(6) == '.' )
        {
            return true;
        }

        return false;
    }

    private static boolean _isGermanShortDate( String sDate_ )
    {
        if ( sDate_.length() == 8
             && sDate_.charAt(2) == '.'
             && sDate_.charAt(5) == '.' )
        {
            return true;
        }

        return false;
    }

    private static boolean _isOldUSDate( String sDate_ )
    {
        if ( sDate_.length() == 10
             && sDate_.charAt(2) == '/'
             && sDate_.charAt(5) == '/' )
        {
            return true;
        }

        return false;
    }

    private static boolean _isOldUSShortDate(String sDate_) {
        if ( sDate_.length() == 8
             && sDate_.charAt(2) == '/'
             && sDate_.charAt(5) == '/' )
        {
            return true;
        }

        return false;
    }

    public static String normalizeDate(String sDate_) {
        String sRetVal = sDate_;

        if ( _isGermanShortDate( sRetVal ) )
        {
            int year = Util.atoi( sRetVal.substring( 6 ) );
            if ( year < 70 )
            {
                year += 100;
            }
            year += 1900;
            sRetVal = Util.itoa( year )
                    + "-"
                    + sRetVal.substring( 3, 5 )
                    + "-"
                    + sRetVal.substring( 0, 2 );
        }
        else if ( _isOldUSShortDate( sRetVal ) )
        {
            int year = Util.atoi( sRetVal.substring( 6 ) );
            if ( year < 70 )
            {
                year += 100;
            }
            year += 1900;
            sRetVal = Util.itoa( year )
                    + "-"
                    + sRetVal.substring( 0, 2 )
                    + "-"
                    + sRetVal.substring( 3, 5 );
        }
        else if ( _isOldUSDate( sRetVal ) )
        {
            sRetVal = sRetVal.substring( 6 )
                    + "-"
                    + sRetVal.substring( 0, 2 )
                    + "-"
                    + sRetVal.substring( 3, 5 );
        }
        else if ( _isGermanDate( sRetVal ) )
        {
            sRetVal = sRetVal.substring( 8 )
                    + "-"
                    + sRetVal.substring( 4, 6 )
                    + "-"
                    + sRetVal.substring( 0, 2 );
        }

        return sRetVal;
    }

    public void export()
    {
        String sStartDate = "";
        String sEndDate   = "x";

        Init init = _pControllable.getInit();
        String sTimezoneID = init.getKeyValue( "TimezoneID" );

        PeriodeDialog periodeDialog = new PeriodeDialog( _pControllable );
        periodeDialog.setVisible( true );
        if ( periodeDialog.isOK() )
        {
            sStartDate = periodeDialog.getStartDate();
            sEndDate = periodeDialog.getEndDate();
            if ( Util.isEmpty( sEndDate ) )
            {
                sEndDate = "x";
            }
        }
        else
        {
            return;
        }
        Util.debug( this, "export().sStartDate: " + sStartDate );
        Util.debug( this, "export().sEndDate:   " + sEndDate   );

        // init stuff
        if ( Util.isEmpty( _sLastDirectory ) )
        {
            _sLastDirectory  = _pControllable.getInit().getFilePath();
            _sLastExportFile = _pControllable.getInit().getFileName().toLowerCase() + ".dat";
        }

        //
        // get file name and check it
        //
        String sExportFile = SwingUtil.getFileName
               ( (JFrame) _pControllable
                 , "Export Timelog Data"
                 , _sLastExportFile
                 , _sLastDirectory
                 , "Export" );
        Util.debug( this, "export().sExportFile: " + sExportFile );
        boolean bFileSavable =
            SwingUtil.isFileSavable((JFrame) _pControllable, sExportFile);
        Util.debug(this, "export().bFileSavable: " + bFileSavable);
        if ( !bFileSavable )
        {
            return;
        }

        _sLastDirectory  = FileUtil.getDir( sExportFile );
        _sLastExportFile = sExportFile;

        export( sExportFile, sStartDate, sEndDate );
    }

    public void export( String sExportFile_
                        , String sStartDate_
                        , String sEndDate_ )
    {
        //
        // create data to export
        //
        StringBuffer sbExport = new StringBuffer(10000);
        StringBuffer sbTable = new StringBuffer(10000);
        StringBuffer sbSummary = new StringBuffer(1000);
        {
            sbExport.append( "Timesheet\t" );
            sbExport.append( _pControllable.getInit().getFileName() );
            sbExport.append( "\n\n" );

            // summary ini
            int aSummary[] = new int[ _pControllable.getProcessSize() ];
            for ( int phase = 0;
                  phase < _pControllable.getProcessSize();
                  phase++ )
            {
                aSummary[ phase ] = 0;
            }

            /*{
                long alTotal = 0;
                int phase = 0;
                for(Enumeration e = _pControllable.getProcessElements();
                    e.hasMoreElements(); )
                {
                    sbExport.append( (String)e.nextElement() + ":\t" );
                    long lSeconds = _aalSummary[phase][Timelog.SUM_TOTAL];
                    alTotal += lSeconds;
                    sbExport.append( SummaryView.formatTime( lSeconds ) ).append( '\n' );
                    phase++;
                }
                sbExport.append( "Total:\t" );
                sbExport.append( SummaryView.formatTime( alTotal ) ).append( "\n\n" );
                }*/

            Enumeration eSessions = _vSessions.elements();
            while ( eSessions.hasMoreElements() )
            {
                int sessionIndex = 0;
                int delta = 0;
                for ( Enumeration eElements = 
                             ((Vector) eSessions.nextElement()).elements();
                      eElements.hasMoreElements(); )
                {
                    String sNextElement = (String) eElements.nextElement();

                    // reformat date
                    if ( sessionIndex == 0 )
                    {
                        sNextElement = normalizeDate( sNextElement );
                        Util.debug( this
                                    , "export(..).cmpStart: "
                                      + sNextElement.compareTo( sStartDate_ ) );
                        Util.debug( this
                                    , "export(..).cmpEnd:   "
                                      + sNextElement.compareTo( sEndDate_ ) );
                        if ( sNextElement.compareTo( sStartDate_ ) < 0 )
                        {
                            break;
                        }
                        if ( sNextElement.compareTo( sEndDate_ ) >= 0 )
                        {
                            break;
                        }
                    }

                    // add to summary
                    {
                        if ( sessionIndex == 4 )
                        {
                            // delta
                            String sHours   = sNextElement.substring( 0, 2 );
                            String sMinutes = sNextElement.substring( 3, 5 );
                            String sSeconds = sNextElement.substring( 6    );

                            delta = Util.atoi( sHours ) * 60 * 60
                                   + Util.atoi( sMinutes ) * 60
                                   + Util.atoi( sSeconds );
                        }
                        if ( sessionIndex == 5 )
                        {
                            // phase
                            aSummary[ Util.atoi( sNextElement ) ] += delta;
                        }
                    }

                    // process phase id to plain text description
                    if ( sessionIndex == 5 )
                    {
                        sNextElement = _pControllable.getProcessElement( sNextElement );
                    }

                    sbTable.append( sNextElement )
                           .append( "\t" );

                    sessionIndex++;
                }
                if ( sessionIndex > 0 )
                {
                    // != 0 means no break
                    sbTable.append("\n");
                }
            }

            //
            // print out summary
            //
            long lTotal = 0;
            int phase = 0;
            Util.debug( this, "export(..).ENTER_SUMMARY" );
            for ( Enumeration e = _pControllable.getProcessElements();
                  e.hasMoreElements(); )
            {
                Util.debug( this, "export(..).phase: " + phase );
                String sNextPhase = (String) e.nextElement();
                long lSeconds = aSummary[ phase ];
                if ( lSeconds == 0 )
                {
                    phase++;

                    continue;
                }
                sbSummary.append( sNextPhase ).append( ":\t" );
                sbSummary.append( SummaryView.formatTime( lSeconds ) ).append( "\n" );
                lTotal += lSeconds;
                phase++;
            }
            sbSummary.append( "Total:\t" );
            sbSummary.append( SummaryView.formatTime( lTotal ) ).append( "\n\n" );
            sbExport.append( sbSummary );

            sbExport.append( "Date\tStart\tStop\tInterrupt\tDelta\tPhase\tComment\n" );
            sbExport.append( sbTable );
        }

        //
        // save stuff
        //
        try
        {
            Util.debug( this, "export(..).sExportFile_: " + sExportFile_ );
            Util.debug( this, "export(..).sbExport: " + sbExport );
            FileUtil.writeFile( sExportFile_, sbExport.toString() );
        }
        catch ( Exception pException )
        {
            Util.printlnErr( "psp.timelog.Timelog.export(..).pException: " + pException );
            SwingUtil.showMessage
                   ( (JFrame) _pControllable
                     , "Error: export file\n\n\""
                       + sExportFile_
                       + "\"\n\n could not be saved!" );
        }
    }

    public void save()
    {
        Util.debug( "Timelog.save().START: " + new Date() );

        // Process speichern
        StringBuffer sTimelog = new StringBuffer( 10000 );
        sTimelog.append(_pControllable.getProcess()).append("\n");

        // Sessions mit phasen als zahl speichern
        for ( Enumeration eSessions = _vSessions.elements();
              eSessions.hasMoreElements(); )
        {
            for ( Enumeration eElements = ((Vector) eSessions.nextElement()).elements();
                  eElements.hasMoreElements(); )
            {
                sTimelog.append((String) eElements.nextElement())
                        .append("\t");
            }
            sTimelog.append("\n");
        }
        Util.debug("Timelog.save().1: " + new Date());
        try
        {
            FileUtil.writeFileWithBackup
                   ( _pControllable.getInit().getFileFullName()
                     , sTimelog.toString() );
        }
        catch ( Exception e )
        {
            SwingUtil.showMessage( (JFrame) _pControllable, "Error: Data File could not be saved!");
        }
        Util.debug("Timelog.save().END: " + new Date());
    }

    public void load(String sTimelog) {
        _clear();
        Vector vSessionStrings = Util.stringToLines(sTimelog);
        for ( Enumeration e = vSessionStrings.elements();
              e.hasMoreElements(); )
        {
            String sSession = (String) e.nextElement();
            _addSession(sSession, false);
        }
        setChanged();
        notifyObservers();
    }

    public void selectSession(int nr_) {
        Util.panicIf(nr_ >= _vSessions.size());
        _selectedSession = nr_;
    }

    public void showSession(int nr_) {
        selectSession(nr_);
        _pSession.set((Vector) _vSessions.elementAt(nr_));
    }

    public String[][] getSessions() {
        return getSessions( true );
    }

    public final long[][] getSummary() {
        _setDaily();
        _setWeekly();
        return _aalSummary;
    }

    public String getLastDate() {
        String sTimezoneID = _pControllable.getInit()
                                           .getKeyValue( Controllable.S_INIT_TIMEZONE_ID );
        String sRetVal = Util.getDate( Util.getCalendar( sTimezoneID ) );
        if (_vSessions != null && _vSessions.size() > 0) {
            int session = _selectedSession;
            if (session == NOTHING_SELECTED) {
                session = _vSessions.size() - 1;
            }
            sRetVal = (String) (((Vector) (_vSessions.elementAt( session ))
                                 )
                                .elementAt( Session.DATE )
                                );
        }

        return sRetVal;
    }

    public String getLastStart() {
        String sTimezoneID = _pControllable.getInit    ()
                                           .getKeyValue( Controllable.S_INIT_TIMEZONE_ID );
        String sRetVal = Util.getTime( Util.getCalendar( sTimezoneID ) );
        if (_vSessions != null && _vSessions.size() > 0) {
            int session = _selectedSession;
            if (session == NOTHING_SELECTED) {
                session = _vSessions.size() - 1;
            }
            sRetVal = (String) (((Vector) (_vSessions.elementAt(session)))
                                .elementAt(Session.START));
        }

        return sRetVal;
    }

    public String getLastStop() {
        String sTimezoneID = _pControllable.getInit    ()
                                           .getKeyValue( Controllable.S_INIT_TIMEZONE_ID );
        String sRetVal = Util.getTime( Util.getCalendar( sTimezoneID ) );
        if (_vSessions != null && _vSessions.size() > 0) {
            int session = _selectedSession;
            if (session == NOTHING_SELECTED) {
                session = _vSessions.size() - 1;
            }
            sRetVal = (String) (((Vector) (_vSessions.elementAt(session)))
                                .elementAt(Session.STOP));
        }

        return sRetVal;
    }

    public String getLastComment() {
        String sRetVal = "";
        if (_vSessions != null && _vSessions.size() > 0) {
            int session = _selectedSession;
            if (session == NOTHING_SELECTED) {
                session = _vSessions.size() - 1;
            }
            sRetVal = (String) (((Vector) (_vSessions.elementAt(session)))
                                .elementAt(Session.COMMENT));
        }

        return sRetVal;
    }

    public int getSelectedIndex() {
        return _selectedSession;
    }
}
