/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package psp.timelog;

import ccl.swing.CCLTable;
import ccl.util.Util;
import java.awt.BorderLayout;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Observable;
import java.util.Observer;
import javax.swing.JPanel;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import psp.timelog.table.TimelogModel;
import psp.timelog.table.TimelogTable;

/**
 * @author <a href="http://www.kclee.com/clemens/">
 *         Chr. Clemens Lee</a>
 *         (<a href="mailto:clemens@kclee.com">
 *         clemens@kclee.com
 *         </a>), Kenneth Westelinck (kenneth.westelinck@pandora.be)
 */
public class TimelogView extends JPanel implements Observer
{
    private static String[] header =
    {
        "Date", "Start", "Stop", "Interrupt", "Delta", "Phase", "Comment"
    };
    private TimelogModel timeLogModel = null;
    private Timelog timeLog = null;
    private Controller controller = null;
    private ListSelectionModel selectionModel = null;
    //private JTable table = null;
    private TimelogTable table = null;

    public TimelogView(Timelog aTimeLog, Controller aController)
    {
        timeLog = aTimeLog;
        timeLog.addObserver(this);
        controller = aController;
        init();
    }

    private void init()
    {
        setLayout(new BorderLayout());

        timeLogModel = new TimelogModel(header);
        table = new TimelogTable(timeLogModel, Controller.getStandardFont());
        table.setAutoResizeMode(CCLTable.AUTO_SIZE_LAST_COLUMN_TO_FIT);
        table.initColumnSizes();
        add(table);

        timeLogModel.setData(getSessions());

        table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        selectionModel = table.getSelectionModel();

        // Use the selection model for the track list
        TimeLogSelectionListener listener = new TimeLogSelectionListener();
        selectionModel.addListSelectionListener(listener);

        table.addMouseListener(listener);
    }

    private String[][] getSessions()
    {
        String[][] aasSession = timeLog.getSessions();

        return aasSession;
    }

    public void update(Observable ov, Object o)
    {
        timeLogModel.clear();
        timeLogModel.setData(getSessions());
        timeLogModel.update();

        int selectedIndex = timeLog.getSelectedIndex();
        if (selectedIndex != Timelog.NOTHING_SELECTED)
        {
            selectionModel.setSelectionInterval( selectedIndex, selectedIndex );
            table.scrollRectToVisible( table.getCellRect( selectedIndex
                                                          , 0
                                                          , true ) );
        }
    }

    private void mousClicked()
    {
    }

    private void mousePressed()
    {
    }

    private class TimeLogSelectionListener implements ListSelectionListener
                                                      , MouseListener
    {
        private int clickCount = 0;
        private int selectedIndex = -1;

        public void mouseClicked(MouseEvent evt)
        {
            clickCount = evt.getClickCount();
            handleSelection();
        }

        public void mousePressed(MouseEvent evt)
        {
        }

        public void mouseReleased(MouseEvent evt)
        {
        }

        public void mouseEntered(MouseEvent evt)
        {
        }

        public void mouseExited(MouseEvent evt)
        {
        }

        /**
         * Handle the ListSelectionEvent.
         *
         * @param evt the event
         */
        public void valueChanged(ListSelectionEvent evt)
        {
            // Ignore extra messages.
            if (evt.getValueIsAdjusting())
            {
                return;
            }

            ListSelectionModel lsm = (ListSelectionModel) evt.getSource();
            selectedIndex = lsm.getMinSelectionIndex();

            // If a row was selected.
            if (!lsm.isSelectionEmpty())
            {
                Util.debug( "TimelogView: update: selectedIndex: "
                            + selectedIndex );
                handleSelection();
            }
        }

        /**
         * Handle the selection. Is called when a row is clicked in the table.
         * If the row is double clicked, we copy the selected session to the
         * input fields.
         * If the row is single clicked, the selected session is merely
         * selected, not copied.
         */
        private void handleSelection()
        {
            if ((clickCount >= 2) && (selectedIndex >= 0))
            {
                timeLog.showSession(selectedIndex);
            }
            else if (selectedIndex >= 0)
            {
                timeLog.selectSession(selectedIndex);
            }

            clickCount = 0;
        }
    }
}
