/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package psp.timelog.table;

import java.util.Vector;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

/**
 * This is a default table model.
 *
 * @author Kenneth Westelinck (kenneth.westelinck@pandora.be)
 * @version $Id: DefaultTableModel.java,v 1.1 2003/05/03 12:05:30 c1emens Exp $
 *         , created on: 2003-02-24
 */
public abstract class DefaultTableModel implements TableModel
{
    private Vector tableModelListeners = new Vector();
    private static String[] header = null;
    private String[][] data = null;

    /** Create a new model. */
    public DefaultTableModel() {
    }
    
    public abstract String[] getLongValues();

    /**
     * Create a new model.
     *
     * @param newHeader the header for the table
     */
    public DefaultTableModel(String[] newHeader) {
        header = new String[newHeader.length];
        for (int i = 0; i < newHeader.length; i++) {
            header[i] = newHeader[i];
        }
    }

    /**
     * Set the data for this model.
     * 
     * @param newData the data
     */
    public void setData(String[][] newData) {
        data = newData;
    }

    /**
     * Get a row from this table.
     *
     * @param index the index for the row
     */
    public Object getRowAt(int index) {
        return data[index];
    }

    /**
     * Get the number of rows.
     * 
     * @return the number of rows.
     */
    public int getRowCount() {
        if (data != null) {
            return data.length;
        } else {
            return 0;
        }
    }

    /**
     * Get the number of columns.
     * 
     * @return the number of columns.
     */
    public int getColumnCount() {
        if (header != null) {
            return header.length;
        } else {
            return 0;
        }
    }

    /**
     * Set the value at a given row and column.
     * 
     * @param row the row
     * 
     * @param column the column
     * 
     * @param o the object to set at the given row and column
     */
    abstract public void setValueAt(Object o, int row, int column);

    /**
     * Get the value from a given row and column.
     *  
     * @param row the row
     * 
     * @param column the column
     * 
     * @return the value as an object
     */
    abstract public Object getValueAt(int row, int column);

    /**
     * Get the class for a given column.
     * 
     * @param the column
     * 
     * @return the class
     */
    abstract public Class getColumnClass(int column);

    /**
     * Get the name for a specified column.
     * 
     * @param column the index of the column
     */
    public String getColumnName(int column) {
        return header[column];
    }

    /**
     * Check if this cell is editable.
     * 
     * @param rowIndex the index for the row
     * 
     * @param columnIndex the index for the column
     * 
     * @return we always return true
     */
    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return true;
    }

    /**
     * Add a listener to this table.
     * 
     * @param listener the listener 
     */
    public void addTableModelListener(TableModelListener listener) {
        tableModelListeners.addElement(listener);
    }

    /**
     * Remove a listener from this table.
     * 
     * @param listener the listener
     */
    public void removeTableModelListener(TableModelListener listener) {
        tableModelListeners.removeElement(listener);
    }

    /**
     * Update this model to reflect the recent changes.
     */
    public void update() {
        int len = tableModelListeners.size();
        TableModelEvent e = new TableModelEvent(this);
        for (int i = 0; i < len; i++) {
            (
                (TableModelListener) tableModelListeners.elementAt(
                    i)).tableChanged(
                e);
        }
    }

    /** Clear this model. */
    public void clear() {
        data = null;
    }
}
