/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package psp.timelog.table;

/**
 * This is the model behind the TimelogView.
 *
 * @author Kenneth Westelinck (kenneth.westelinck@pandora.be)
 *         , Chr. Clemens Lee (clemens@kclee.com)
 * @version $Id: TimelogModel.java,v 1.1 2003/05/03 12:05:30 c1emens Exp $
 *         , created on: 2003-03-04
 */
public class TimelogModel extends DefaultTableModel
{
    private final String[] longValues =
    {
        "x_XXXX-XX-XX",
        "x_XX:XX:XX",
        "x_XX:XX:XX",
        "x_XX:XX:XX",
        "x_XX:XX:XX",
        "x_F/Compile",
        "x_XXXXXXXXXX" };

    public TimelogModel(String[] header) {
        super(header);
    }
    
    public String[] getLongValues() {
        return this.longValues;
    }

    /**
     * Get the value from a given row and column.
     *  
     * @param row the row
     * 
     * @param column the column
     * 
     * @return the value as an object
     */
    public Object getValueAt(int row, int column) {
        return ((String[]) super.getRowAt(row))[column];
    }
    
    public void setValueAt(Object o, int row, int column) {
        // not implemented
    }
    
    /**
     * Check if this cell is editable.
     * 
     * @param rowIndex the index for the row
     * 
     * @param columnIndex the index for the column
     * 
     * @return we always return true
     */
    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return false;
    }

    /**
     * Get the class for a given column.
     * 
     * @param the column
     * 
     * @return String.class, all the values are of type String
     */
    public Class getColumnClass(int column) {
        return String.class;
    }
}
