package psp.timelog.table;

import ccl.swing.CCLTable;
import java.awt.Component;
import java.awt.Font;
import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

/**
 * Table class used through out the timelog application.
 *
 * @author Kenneth Westelinck (kenneth.westelinck@pandora.be)
 *         , Chr. Clemens Lee (clemens@kclee.com)
 * @version $Id: TimelogTable.java,v 1.1 2003/05/03 12:05:30 c1emens Exp $
 */
public class TimelogTable extends CCLTable
{

    /**
     * Standard constructor.
     * 
     * @param aModel    the table model to use.
     * @param font   the font to use for the table header.
     */
    public TimelogTable(DefaultTableModel aModel, Font font) {
        super(aModel, font);
    }

    /**
     * Initialize the widths for the columns to match the width of the data
     * contained, or the width of the header.
     */
    public void initColumnSizes() {
        JTable table = getTable();
        int resizeMode = getResizeMode();
        initColumnSizes( table, resizeMode );   
    }

    /**
     * Initialize the widths for the columns to match the width of the data
     * contained, or the width of the header.
     */
    public void initColumnSizes(JTable table, int resizeMode) {
        DefaultTableModel model = (DefaultTableModel)table.getModel();
        TableColumn column = null;
        Component comp = null;
        int headerWidth = 0;
        int cellWidth = 0;
        int columnWidth = 0;
		
        TableCellRenderer headerRenderer =
               table.getTableHeader().getDefaultRenderer();
		
        for (int i = 0; i < model.getLongValues().length; i++) {
            column = table.getColumnModel().getColumn(i);
		
            comp =
                   headerRenderer.getTableCellRendererComponent(
                                                                null,
                                                                column.getHeaderValue(),
                                                                false,
                                                                false,
                                                                0,
                                                                0);
            headerWidth = comp.getPreferredSize().width;
		
            comp =
                   table.getDefaultRenderer(
                                            model.getColumnClass(i)).getTableCellRendererComponent(
                                                                                                   table,
                                                                                                   model.getLongValues()[i],
                                                                                                   false,
                                                                                                   false,
                                                                                                   0,
                                                                                                   i);
            cellWidth = comp.getPreferredSize().width;
            columnWidth = cellWidth;
		
            // If AUTO_SIZE_LAST_COLUMN_TO_FIT is set, don't size the last
            // column.
            if ((i < model.getLongValues().length - 1)
                || (resizeMode != AUTO_SIZE_LAST_COLUMN_TO_FIT)) {
                column.setPreferredWidth(columnWidth);
                column.setMinWidth(columnWidth);
                column.setMaxWidth(columnWidth);
            }
		
            column.setResizable(false);
        }		
    }
}
