/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Copyright: Chr. Clemens Lee
 */

package psp.timelog.test;

import ccl.util.FileUtil;
import ccl.util.Test;
import ccl.util.Util;
import java.awt.Container;
import java.io.File;
import java.util.Vector;
import javax.swing.JViewport;
import psp.timelog.Controller;
import psp.timelog.SummaryView;
import psp.timelog.Timelog;

/**
 * @version $Id: TimelogTest.java,v 1.3 2003/05/03 12:13:00 c1emens Exp $
 * @author  Chr. Clemens Lee
 */
public class TimelogTest extends Test 
{
    /**
     * Starts a new timelog application with a clean ini file.
     *
     * @param   sDataFile_   a timelog project file name.
     */
    private Controller startTimelog( String sDataFile_ )
    {
        String sLocalPath = FileUtil.getClassPath(this);
        final String sIniFile = FileUtil.concatPath( sLocalPath,
                                                     "timelogtest.ini" );

        FileUtil.copy( FileUtil.concatPath( sLocalPath, ".." ) +
                       File.separator + "default.ini",
                       sIniFile );

        final String[] asArg = { "-inifile",
                                 sIniFile,
                                 sDataFile_ };
        
        final Controller controller = new Controller();
        controller.initialize( asArg, null );
        Runnable runTimelog = new Runnable() {
                public void run() {
                    controller.start();
                }
            };
        new Thread( runTimelog ).start();

        Util.sleep( 6 );

	return controller;
    }

    /**
     * Tests export feature.
     */
    private void _testExport( final String sDataFile_,
                                    String sStartDate_,
                                    String sEndDate_,
                                    String sExportFile_ )
    {
        Controller controller = startTimelog( sDataFile_ );

        Timelog timelog = controller.getTimelog();
        String sLocalPath = FileUtil.getClassPath(this);
        String sExportFile = FileUtil.concatPath
               ( sLocalPath, "exporttest.dat" );
        FileUtil.delete( sExportFile_ );
        timelog.export( sExportFile_, sStartDate_, sEndDate_ );

        bugIf( !FileUtil.existsFile( sExportFile ), "File " +
               sExportFile + " doesnt exist!" );

	controller.setExit();
    }

    /**
     * Regression test to track down a bug which doesn't let us open a
     * new project if there has been already a session in the current
     * project.
     */
    public void testNewProject()
	throws Exception
    {
	// open a project with a couple of sessions
        String sLocalPath = FileUtil.getClassPath(this);
        String sTimelogFile = FileUtil.concatPath( sLocalPath,
                                                   "worktest.timelog" );
        Controller controller = startTimelog( sTimelogFile );

	// open a new project
	String sTempDir = FileUtil.getTempDir();
	controller.open( FileUtil.concatPath( sTempDir, "_temp_new.timelog" )
			 , "PSP1.1" );

	// check that no exception has been thrown 
	// if you reach here it has been already successfull
	Assert( true );

	// close application
	controller.setExit();
    }

    /**
     * Checks the width of the summary view.
     * It should be exactly the sum of all columns.
     */
    public void testSummaryViewWidth()
	throws Exception
    {
	// open a project with a couple of sessions
        String sLocalPath = FileUtil.getClassPath(this);
        String sTimelogFile = FileUtil.concatPath( sLocalPath,
                                                   "worktest.timelog" );
        Controller controller = startTimelog( sTimelogFile );

        SummaryView summaryView = controller.getSummaryView();
        // summary view swing structure:
        // - ScrollableJTable
        //   - viewport
        //     - jtable
        //   - viewpoert
        //     - jtable header
        // viewport and table should be the same size
        /*Util.println( ccl.swing.SwingUtil.getContainerDump( summaryView ) );*/
        Container container = (Container) summaryView.getComponent( 0 );
        //container = (Container) container.getComponent( 0 );
        JViewport viewport = (JViewport) container.getComponent( 0 );
        Container table = (Container) viewport.getComponent( 0 );
        Assert( viewport.getSize().width == table.getSize().width
                , "Viewport width: " + viewport.getSize().width + ", table width: " + table.getSize().width );

        /*int width = summaryView.getSize().width;
          Util.println( "Width: " + width );*/

	// close application
	controller.setExit();
    }

    public TimelogTest() {
        super();
    }

    public TimelogTest( Test pTest_ ) {
        super( pTest_ );
    }

    protected void _doIt() 
        throws Exception
    {
        testSummaryViewWidth();
        /*
        Test test = new SummaryViewTest( this );
        test.run();
        setTests( test );
        */

	testNewProject();

        final String sLocalPath = FileUtil.getClassPath(this);
        final String sIniFile = FileUtil.concatPath( sLocalPath,
                                                     "timelogtest.ini" );

        FileUtil.copy( FileUtil.concatPath( sLocalPath, ".." ) +
                       File.separator + "default.ini",
                       sIniFile );

        final String sTestFile = FileUtil.concatPath( sLocalPath,
                                                      "test.timelog" );

        final String[] asArg = { "-inifile",
                                 sIniFile,
                                 sTestFile };
        
        final Controller controller = new Controller();
        controller.initialize( asArg, null );
        Runnable runTimelog = new Runnable() {
                public void run() {
                    //controller.start();
                }
            };
        new Thread( runTimelog ).start();

        Util.sleep( 6 );

        Timelog timelog = controller.getTimelog();
        String sExportFile = FileUtil.concatPath
               ( sLocalPath, "exporttest.dat" );
        FileUtil.delete( sExportFile );
        timelog.export( sExportFile, "", "x" );

        bugIf( !FileUtil.existsFile( sExportFile ) );

        controller.exit();

        FileUtil.delete( sExportFile );
        String sTimelogFile = FileUtil.concatPath( sLocalPath,
                                                   "worktest.timelog" );
        _testExport( sTimelogFile, "1999-10-25", "1999-11-22", sExportFile );

        bugIf( !FileUtil.existsFile( sExportFile ), "File " +
               sExportFile + " doesnt exist!" );

        String sExportContent = FileUtil.readFile
               ( sExportFile );
        Vector vExport = Util.stringToLines( sExportContent );
        bugIf( vExport.size() < 10, "vExport.size: " + 
               vExport.size() );
    }

    public static void main( String[] asArg_ ) {
        Test pTest = (Test)(new TimelogTest());
        pTest.setVerbose( true );
        pTest.run();
        pTest.printResult();

        System.exit( 0 );
    }
}
