/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package psp.timelog.util;

import java.util.Calendar;

import psp.timelog.Timelog;
import ccl.util.Util;

/**
 * @author <a href="http://www.kclee.com/clemens/">
 *         Chr. Clemens Lee</a>
 *         &lt;<a href="mailto:clemens@kclee.com">
 *         clemens@kclee.com
 *         </a>>
 */
public class ObservableShortDate extends ObservableValue {
    private Calendar _pCalendar = null;

    public ObservableShortDate() {
        super();

        _pCalendar = Util.getCalendar( "GMT" );
    }

    public ObservableShortDate(String sDate_) {
        super();

        _pCalendar = Util.getCalendar( "GMT" );
        set( sDate_ );
    }
    
    public void set(String sDate_) {
        String sDate = Timelog.normalizeDate( sDate_ );
        _pCalendar.set( Util.atoi( sDate.substring( 0, 4 ) ),
                        Util.atoi( sDate.substring( 5, 7 ) ) - 1,
                        Util.atoi( sDate.substring( 8 ) ) );

        setChanged();
        notifyObservers();
    }

    /**
     * @return   e.g. 11/17/99
     */
    public String getShortDate() {
        String sRetVal = Util.getDate( _pCalendar );
        Util.debug( this, "toDateString().sRetVal: " +
                    sRetVal );
        sRetVal = sRetVal.substring( 5, 7 ) + "/" +
               sRetVal.substring( 8, 10 ) + "/" +
               sRetVal.substring( 2, 4 );

        return sRetVal;
    }
    
    public String getDate() {
        return Util.getDate( _pCalendar );
    }

    public String toString() {
        return getShortDate();
    }
}
