/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package psp.timelog.util;

import java.util.Calendar;

import ccl.util.Util;

/**
 * This class represents a time value object.
 *
 * @author <a href="http://www.kclee.com/clemens/">
 *         Chr. Clemens Lee</a>
 *         &lt;<a href="mailto:clemens@kclee.com">
 *         clemens@kclee.com
 *         </a>>, Kenneth Westelinck (kenneth.westelinck@pandora.be)
 */
public class ObservableTime extends ObservableValue {
    private Calendar _pCalendar = null;

    public ObservableTime(String sTime_) {
        super();

        _pCalendar = Util.getCalendar();
        Util.setTime( _pCalendar, sTime_ );
    }

    public void set(String sTime_) {
        Util.panicIf( Util.isEmpty( sTime_ ) );

        if (sTime_.length() == 2) 
        {
            Util.setTime(_pCalendar, sTime_ + ":00:00");
        }
        else if ( sTime_.length() == 5 ) 
        {
            Util.setTime( _pCalendar, sTime_ + ":00" );
        } else {
            Util.panicIf( sTime_.length() != 8 );
            Util.setTime( _pCalendar, sTime_ );
        }

        setChanged();
        notifyObservers();
    }
    
    public String toString() {
        return Util.getTime( _pCalendar );
    }
}
