/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package psp.timelog.workstats;

import ccl.util.Exitable;
import ccl.util.FileUtil;
import ccl.util.Init;
import ccl.util.Util;
import java.util.Calendar;
import java.util.Date;
import java.util.Enumeration;
import java.util.GregorianCalendar;
import java.util.Vector;
import psp.timelog.Controllable;
import psp.timelog.Session;
import psp.timelog.SummaryView;
import psp.timelog.Timelog;

/**
 * Use cases:
 * 1. Get netto work time of today
 * 2. Get netto work time of this week
 * 3. Get netto work time of this month
 *
 * @author  Chr. Clemens Lee (clemens@kclee.com)
 * @version $Header: /cvsroot/time1og/timelog/src/psp/timelog/workstats/Main.java,v 1.1 2003/03/30 12:39:24 c1emens Exp $
 */
public class Main implements Exitable {
    private static Init _pInit = null;
    private static String _sProcess = null;
    private static Vector _vProcess = null;
    private static Vector _vsDeadlineDates = null;

    private boolean _bExit = false;

    private Main() {
        super();
    }

    private static void open( Timelog pTimelog_, 
                              String sFileFullName_ )
    {
        if ( !FileUtil.existsFile(sFileFullName_)) {
            Util.println( "File '" +
                          sFileFullName_ +
                          "' does not exist!" );

            System.exit( 1 );
        }
		
        // timelog laden
        String sTimelog = "";
        try {
            sTimelog = FileUtil.readFile( sFileFullName_ );
        } catch(Exception e) {
            Util.println( "Error while reading file '" +
                          sFileFullName_ +
                          "': " + e );
            
            System.exit( 1 );
        }
        
        // extract process
        int index = sTimelog.indexOf('\n');
        if (index == -1) {
            Util.println( "Error in Timelog File!" );
            
            System.exit( 1 );
        }
        _sProcess = sTimelog.substring(0, index);
        sTimelog = sTimelog.substring( index + 1, sTimelog.length() );
        String sProcessElements = _pInit.getKeyValue( LocalController.S_INIT_PROCESS, _sProcess );
        _vProcess = Util.stringToLines( sProcessElements, 
                                        LocalController.C_PROCESS_DELIMITER );

        pTimelog_.load( sTimelog );
    }

    private static long _getRounded( long lSeconds_ ) {
        long lRetVal = ((lSeconds_ + (15*60))/(30*60))*(30*60);

        return lRetVal;
    }

    private static long _getPocket( long lSeconds_ ) {
        long lRounded = _getRounded( lSeconds_ );

        return lSeconds_ - lRounded;
    }

    private static void _printlnPocket( long lPocket_ ) {
        String sSign = "-";
        if ( lPocket_ >= 0 ) {
            sSign = "+";
        }

        Util.println( sSign + Util.paddWithZero
                      ( Math.abs( (int)lPocket_/60 ), 
                        2 ) +
                      " m)" );
    }

    /**
     * @return lMonthPocket
     */
    private static long _printMonth( String sLastMonth_, long lMonth_, long lMonthPocket_ ) {
        Util.print( "\nMonth " + sLastMonth_ + ":    " + 
                    SummaryView.formatTime( lMonth_ ) );

        long lMonthRounded = _getRounded( lMonth_ + lMonthPocket_ );
        Util.print( "  (" + 
                    SummaryView.formatTime( lMonthRounded ) +
                    " " );

        long lNewMonthPocket = _getPocket( lMonth_ + lMonthPocket_ );
        _printlnPocket( lNewMonthPocket );

        Util.println();

        return lNewMonthPocket;
    }

    private static void _loadDeadlines( String sDeadlinesFile_ ) {
        String sDeadlines = null;

        try {
            sDeadlines = FileUtil.readFile( sDeadlinesFile_ );
        } catch( Exception pException ) {
            Util.printlnErr( "Could not read deadlines file '" +
                             sDeadlinesFile_ +
                             "'!" );

            return;
        }

        _vsDeadlineDates = Util.stringToLines( sDeadlines );
    }

    private static boolean _useDeadlines() {
        return _vsDeadlineDates != null &&
               _vsDeadlineDates.size() > 0;
    }

    private static int _deadline = 0;

    private static String _getNextDeadline() {
        String sRetVal = null;

        if ( !_useDeadlines() ||
             _deadline >= _vsDeadlineDates.size() ) 
        {
            return sRetVal;
        }

        sRetVal = (String)_vsDeadlineDates.elementAt
               ( _deadline );
        _deadline++;

        return sRetVal;
    }

    private static class Accumulator {
        public String sNow     = null;
        public String sLast    = null;
        public long   lCurrent = 0;
        public long   lPocket  = 0;

        public Accumulator() {
            super();
        }
        
        public String toString() {
            return "Accumulator[" + sNow + "," +
                   sLast + "," + lCurrent + "," + 
                   lPocket + "]";
        }
    }

    private static void _printLastDay( Accumulator pAccumulator_ ) {
        if ( pAccumulator_.sLast == null ) {
            pAccumulator_.sLast = pAccumulator_.sNow;
        }
        if ( !pAccumulator_.sNow.equals
             ( pAccumulator_.sLast ) ) 
        {
            Util.print( pAccumulator_.sLast + ":   " + 
                        SummaryView.formatTime
                        ( pAccumulator_.lCurrent ) );
            long lTodayRounded = _getRounded
                   ( pAccumulator_.lCurrent + 
                     pAccumulator_.lPocket );
            pAccumulator_.lPocket = _getPocket
                   ( pAccumulator_.lCurrent + 
                     pAccumulator_.lPocket );

            // print total rounded
            Util.print( "   (" );
            if ( lTodayRounded == 0 ) {
                Util.print( "00 m 00 h" );
            } else {
                Util.print( SummaryView.formatTime( lTodayRounded ) );
            }
            Util.print( " " );

            _printlnPocket( pAccumulator_.lPocket );
            
            pAccumulator_.lCurrent = 0;
            pAccumulator_.sLast = pAccumulator_.sNow;
        }
    }

    private static void _processDeadline( String sToday_,
                                          Accumulator pAccumulator_ ) 
    {
        if ( _useDeadlines() ) {
            // if now > deadline -> print
            if ( sToday_.compareTo( pAccumulator_.sNow ) >= 0 ) {
                Util.print( "Periode\n" + 
                            pAccumulator_.sLast +
                            "-\n" +
                            pAccumulator_.sNow +
                            ":  " + 
                            SummaryView.formatTime
                            ( pAccumulator_.lCurrent ) );
                long lRounded = _getRounded
                       ( pAccumulator_.lCurrent +
                         pAccumulator_.lPocket );
                pAccumulator_.lPocket = _getPocket
                       ( pAccumulator_.lCurrent + 
                         pAccumulator_.lPocket );
                Util.print( "  (" + 
                            SummaryView.formatTime
                            ( lRounded ) +
                            " " );
                
                _printlnPocket( pAccumulator_.lPocket );
                Util.println();
                
                pAccumulator_.lCurrent = 0;
                pAccumulator_.sLast = pAccumulator_.sNow;
                pAccumulator_.sNow = _getNextDeadline();
            }
        }
    }

    private static void _daily( Timelog pTimelog_ ) {
        String[][] aasSession = pTimelog_.getSessions( false );

        if ( aasSession.length == 0 ) {
            return;
        }

        String sToday = null;
        Accumulator accDaily = new Accumulator();

        long lWeek = 0;
        int lastWeek = -1;
        long lWeekPocket = 0;

        String sMonth = null;
        String sLastMonth = null;
        long lMonth = 0;
        long lMonthPocket = 0;

        Accumulator accDeadline = new Accumulator();
        accDeadline.sLast = _getNextDeadline();
        accDeadline.sNow = _getNextDeadline();

        long lTotal = 0;
        for( int session = 0; session < aasSession.length;
             session++ )
        {
            // get phase
            String sPhase = aasSession[ session ][ Session.PHASE ];
            if ( Util.atoi( sPhase ) >= 10 ) {
                continue;
            }

            // get date
            {
                sToday = aasSession[ session ][ Session.DATE ];
                Util.debug( "date: " + sToday );

                // print day
                accDaily.sNow = sToday;
                _printLastDay( accDaily );

                // week
                {
                    int week = _getWeek( sToday );
                    if ( lastWeek == -1 ) {
                        lastWeek = week;
                    }
                    if ( (week > lastWeek) ||
                         ((lastWeek > 50) && (week < 50)) ) 
                    {
                        Util.println( "             +=========  +=================" );
                        Util.print( "Week " +
                                    Util.paddWithSpace( lastWeek, 2 ) +
                                    ":      " +
                                    SummaryView.formatTime( lWeek ) );
                        long lWeekRounded = _getRounded( lWeek + lWeekPocket );
                        lWeekPocket = _getPocket( lWeek + lWeekPocket );
                        Util.print( "   (" + 
                                    SummaryView.formatTime( lWeekRounded ) +
                                    " " );

                        _printlnPocket( lWeekPocket );
                        Util.println();

                        lWeek = 0;
                        lastWeek = week;
                    }
                    Util.debug( "week: " + week );
                }

                // month
                {
                    sMonth = sToday.substring( 5, 7 );
                    if ( sLastMonth == null ) {
                        sLastMonth = sMonth;
                    }
                    if ( !sMonth.equals( sLastMonth ) ) {
                        lMonthPocket = _printMonth( sLastMonth, lMonth, lMonthPocket );
                        lMonth = 0;
                        sLastMonth = sMonth;
                    }
                }

                // deadline
                _processDeadline( sToday, accDeadline );
            }

            // delta time
            String sDelta = aasSession[ session ][ Session.DELTA ];
            Util.debug( "delta: " + sDelta );
            long lDelta = Util.atoi( sDelta.substring( 0, 2) ) * 3600 +
                   Util.atoi( sDelta.substring( 3, 5 ) ) * 60 +
                   Util.atoi( sDelta.substring( 6 ) );
            lWeek += lDelta;
            lMonth += lDelta;
            accDaily.lCurrent += lDelta;
            accDeadline.lCurrent += lDelta;
            lTotal += lDelta;
        } // next session

        accDaily.sNow = "tomorrow";
        _printLastDay( accDaily );

        if ( lastWeek > -1 ) {
            Util.println( "             +=========  +=================" );
            Util.print( "Week " +
                        Util.paddWithSpace( lastWeek, 2 ) +
                        ":      " +
                        SummaryView.formatTime( lWeek ) );
            
            long lWeekRounded = _getRounded( lWeek + lWeekPocket );
            lWeekPocket = _getPocket( lWeek + lWeekPocket );
            Util.print( "   (" + 
                        SummaryView.formatTime( lWeekRounded ) +
                        " " );
            
            if ( lWeekPocket >= 0 ) {
                Util.println( "+" + Util.paddWithZero
                              ( (int)lWeekPocket/60, 
                                2 ) +
                              " m)" );
            } else {
                Util.println( "-" + Util.paddWithZero
                              ( Math.abs( (int)lWeekPocket/60 ), 
                                2 ) +
                              " m)" );
            }
            Util.println();
        }

        if ( sLastMonth != null ) {
            _printMonth( sLastMonth, lMonth, lMonthPocket );
        }

        _processDeadline( "x", accDeadline );

        Util.println( "Total: " + SummaryView.formatTime( lTotal ) );
    }

    private static String[][] _createSummary( Timelog pTimelog_ ) {
        LocalController pLocalController = new LocalController();
        final long aalSummary[][] = pTimelog_.getSummary();
        int processSize = pLocalController.getProcessSize();
        String aasSummary[][] = new String[ processSize + 1 ][ Timelog.SUM_FIELDS + 1 ];
        final long alTotal[] = new long[ Timelog.SUM_FIELDS ];
        
        int phase = 0;
        for( Enumeration e = pLocalController.getProcessElements();
             e.hasMoreElements(); )
        {
            aasSummary[phase][0] = (String)e.nextElement();
            long lSeconds = aalSummary[phase][Timelog.SUM_TOTAL];
            alTotal[Timelog.SUM_TOTAL] += lSeconds;
            aasSummary[phase][Timelog.SUM_TOTAL + 1] =
                   SummaryView.formatTime(lSeconds);
            aasSummary[phase][2] = "";
            lSeconds = aalSummary[phase][Timelog.SUM_WEEK];
            alTotal[Timelog.SUM_WEEK] += lSeconds;
            aasSummary[phase][Timelog.SUM_WEEK + 1] =
                   SummaryView.formatTime(lSeconds);
            lSeconds = aalSummary[phase][Timelog.SUM_DAY];
            if ( phase < 10 ) {
                alTotal[Timelog.SUM_DAY] += lSeconds;
            }
            aasSummary[phase][Timelog.SUM_DAY + 1] =
                   SummaryView.formatTime(lSeconds);
            phase++;
        }
        aasSummary[processSize][0] = "Total";
        aasSummary[processSize][Timelog.SUM_TOTAL + 1] = SummaryView.formatTime(alTotal[Timelog.SUM_TOTAL]);
        aasSummary[processSize][Timelog.SUM_WEEK + 1] = SummaryView.formatTime(alTotal[Timelog.SUM_WEEK]);
        aasSummary[processSize][Timelog.SUM_DAY + 1] = SummaryView.formatTime(alTotal[Timelog.SUM_DAY]);
        
        return aasSummary;
    }

    private static int _getWeek( String sDate_ ) {
        Calendar pCalendar = Util.getCalendar();
        pCalendar.set( Calendar.YEAR, Util.atoi( sDate_.substring( 0, 4 ) ) );
        pCalendar.set( Calendar.MONTH, Util.atoi( sDate_.substring( 5, 7 ) ) - 1 );
        pCalendar.set( Calendar.DAY_OF_MONTH, Util.atoi( sDate_.substring( 8 ) ) );

        return pCalendar.get( Calendar.WEEK_OF_YEAR );
    }

    private static void _printWeekTimes( String[][] aasSession_ ) {
        if ( aasSession_.length == 0 ) {
            return;
        }

        int lastWeek = -1;
        long lWeek = 0;
        long lTotal = 0;
        for( int session = 0; session < aasSession_.length;
             session++ )
        {
            // get phase
            String sPhase = aasSession_[ session ][ Session.PHASE ];
            if ( Util.atoi( sPhase ) >= 10 ) {
                continue;
            }

            // get date
            {
                String sDate = aasSession_[ session ][ Session.DATE ];
                Util.debug( "date: " + sDate );
                int week = _getWeek( sDate );
                if ( lastWeek == -1 ) {
                    lastWeek = week;
                }
                if ( week > lastWeek ) {
                    Util.println( "Week " +
                                  lastWeek +
                                  ": " +
                                  SummaryView.formatTime( lWeek ) );
                    lWeek = 0;
                    lastWeek = week;
                }
                Util.debug( "week: " + week );
            }

            // delta time
            String sDelta = aasSession_[ session ][ Session.DELTA ];
            Util.debug( "delta: " + sDelta );
            long lDelta = Util.atoi( sDelta.substring( 0, 2) ) * 3600 +
                   Util.atoi( sDelta.substring( 3, 5 ) ) * 60 +
                   Util.atoi( sDelta.substring( 6 ) );
            lWeek += lDelta;
            lTotal += lDelta;
        }
        if ( lastWeek > -1 ) {
            Util.println( "Week " +
                          lastWeek +
                          ": " +
                          SummaryView.formatTime( lWeek ) );
        }

        Util.println( "Total: " + SummaryView.formatTime( lTotal ) );
    }

    public static void main( String[] asArg_ ) {
        Main pMain = new Main();
        _pInit = new Init( pMain, asArg_, "$Header: /cvsroot/time1og/timelog/src/psp/timelog/workstats/Main.java,v 1.1 2003/03/30 12:39:24 c1emens Exp $" );
        if ( pMain._bExit ) {
            System.exit( 0 );
        }

        // get first argument and load data
        Timelog pTimelog = null;
        {
            if ( _pInit.getArguments().size() != 1 ) {
                Util.println( "Usage: workstats work.timelog" );
                
                System.exit( 1 );
            }
            
            String sWorkTimelogFile = (String)_pInit.getArguments().elementAt( 0 );
            Util.debug( "psp.timelog.workstats.Main.main(..).sWorkTimelogFile: " +
                        sWorkTimelogFile );
            
            Session pSession = new Session( _pInit );
            pTimelog = new Timelog( new LocalController(), pSession );

            // load 
            open( pTimelog, sWorkTimelogFile );

        }

        String sDeadlinesFile = (String)_pInit.getOptions().
               get( "deadlines_file" );
        if ( sDeadlinesFile != null ) {
            _loadDeadlines( sDeadlinesFile );
        }
            
        if ( _pInit.getOptions().get( "daily" ) != null ) {
            _daily( pTimelog );
        } else {
            // calculate netto work time this week
            String[][] aasSession = pTimelog.getSessions( false );
            /*Util.println( "Netto work time this week: " +
              _getLastWeekTime( aasSession ) );*/
            _printWeekTimes( aasSession );
            
            // calculate netto work time of today
            String[][] aasSummary = _createSummary( pTimelog );
            
            Util.println( "Today: " +
                          aasSummary[ aasSummary.length - 1][ 3 ] );
        }

        System.exit( 0 );
    }
    
    public void setExit() {
        _bExit = true;
    }    

    public static class LocalController implements Controllable {
        public LocalController() {
            super();
        }

        public Init getInit() {
            Util.debug( this, "getInit().START" );

            return _pInit;
        }

        public int getProcessSize() {
            int retVal = _vProcess.size();
            Util.debug( this, "getProcessSize().retVal: " +
                        retVal );

            return retVal;
        }

        public Enumeration getProcessElements() {
            return _vProcess.elements();
        }

        public String getProcessElement( String sIndex_ ) {
            Util.debug( this, "getProcessElement(..).START" );

            int index = Util.atoi(sIndex_);
            Util.panicIf(index >= _vProcess.size());

            return (String)_vProcess.elementAt( index );
        }

        public void setModified() {
            Util.debug( this, "setModified().START" );
        }

        public String getProcess() {
            Util.debug( this, "getProcess()._sProcess: " +
                        _sProcess );

            return _sProcess;
        }
    }
}
