<?xml version="1.0" encoding="iso-8859-1"?>
<!DOCTYPE xsl:stylesheet [
<!ENTITY nbsp   "&#160;">
<!ENTITY eacute "&#233;">
]>
<xsl:stylesheet
   version="1.0"
   xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <xsl:output method  ="html"
              encoding="iso-8859-1"
              indent  ="no"/>
  <xsl:strip-space elements="*"/>


  <xsl:template match="/">
    <xsl:variable name="project"><xsl:value-of select="String:toLowerCase(String:new(project/name))" xmlns:String="xalan://java.lang.String"/></xsl:variable>
    <html>
      <head>
        <title><xsl:value-of select="project/name"/> Release History</title>
      </head>
      <body text="#000000" bgcolor="#C0C0FF" link="#0000FF" vlink="#800080" alink="#FF0080">
        <center>&nbsp;</center>
        <center><h1><xsl:value-of select="project/name"/> Release History</h1></center>
        <center>&nbsp;</center>

        Continue reading on 
        <a>
          <xsl:attribute name="href">index.html#release</xsl:attribute>
          <xsl:value-of select="project/name"/>
        </a>'s main page.
        <h2>Table of Contents</h2>

        <ul>
          <xsl:apply-templates select="project/version" mode="toc"/>
        </ul>

        <h2>Release History</h2>

        <xsl:apply-templates select="project/version" mode="release"/>

        <xsl:call-template name="p"/>&nbsp;
        <xsl:call-template name="br"/>
        Go back to <a href="index.html#release"><i>Time</i>log</a>.<xsl:call-template name="br"/>
Go to <a href="../index.html">my Java page</a>.
<xsl:text disable-output-escaping="yes">&lt;br&gt;</xsl:text>
<HR WIDTH="100%%" />
<a name="contact" /><A HREF="http://www.kclee.com/clemens/">Chr.
Clemens Lee</A>, email: <i><A HREF="mailto:clemens@kclee.com">clemens@kclee.com</A></i>
      </body>
    </html>
  </xsl:template>

  <xsl:template match="feature|requirement|step|credit|bug">
    <li><xsl:apply-templates/></li>
  </xsl:template>
  
  <xsl:template match="i">
    <i><xsl:apply-templates/></i>
  </xsl:template>

  <xsl:template match="tt">
    <tt><xsl:apply-templates/></tt>
  </xsl:template>

  <xsl:template match="pre">
    <blockquote><pre><xsl:apply-templates/></pre></blockquote>
  </xsl:template>

  <xsl:template match="p" name="p">
        <xsl:text disable-output-escaping="yes">&lt;p&gt;

</xsl:text>
  </xsl:template>

  <xsl:template match="br" name="br">
    <xsl:text disable-output-escaping="yes">&lt;br&gt;
</xsl:text>
  </xsl:template>

  <xsl:template match="projectname">
    <xsl:value-of select="/project/name"/>
  </xsl:template>

  <xsl:template match="zipfile">
    <xsl:call-template name="projectdir"/>
    <xsl:text>.zip</xsl:text>
  </xsl:template>

  <xsl:template match="new">
    <h4>What's New</h4>
    <ol>
      <xsl:apply-templates select="feature"/>
    </ol>
  </xsl:template>

  <xsl:template match="bugs">
    <h4>Bugs Fixed</h4>
    <ol>
      <xsl:apply-templates select="bug"/>
    </ol>
  </xsl:template>

  <xsl:template match="projectdir" name="projectdir">
    <xsl:value-of select="String:toLowerCase(String:new(/project/name))" xmlns:String="xalan://java.lang.String"/>
    <xsl:value-of select="/project/version[position()=1]/major"/><xsl:text>.</xsl:text><xsl:value-of select="/project/version[position()=1]/release"/>
  </xsl:template>

  <xsl:template match="projectnamelowercase">
    <xsl:value-of select="String:toLowerCase(String:new(/project/name))" xmlns:String="xalan://java.lang.String"/>
  </xsl:template>

  <xsl:template match="link">
    <xsl:text disable-output-escaping="yes">&lt;a href="</xsl:text>
    <xsl:if test="@type!=''"><xsl:value-of select="@type"/>:</xsl:if>
    <xsl:value-of select="."/>
    <xsl:if test="@id!=''">
      <xsl:variable name="currentid"><xsl:value-of select="@id"/></xsl:variable>
      <xsl:apply-templates select="//*[@id=$currentid]" mode="link"/>
    </xsl:if>
    <xsl:text disable-output-escaping="yes">"&gt;</xsl:text>
    <xsl:choose>
      <xsl:when test="@text!=''">
        <xsl:value-of select="@text"/>
      </xsl:when>
      <xsl:when test="@id!=''">
        <xsl:variable name="currentid"><xsl:value-of select="@id"/></xsl:variable>
        <xsl:apply-templates select="//*[@id=$currentid and @name!='']" mode="linktext"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="."/>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text disable-output-escaping="yes">&lt;/a&gt;</xsl:text>
  </xsl:template>

  <xsl:template match="link" mode="links">
    <dt><a>
        <xsl:attribute name="href"><xsl:value-of select="url"/></xsl:attribute>
        <xsl:value-of select="name"/>
      </a>
    </dt>
    <dd>
      <xsl:value-of select="description"/>
      <xsl:text> [</xsl:text><xsl:value-of select="date"/><xsl:text>]</xsl:text>
    </dd>
  </xsl:template>

  <xsl:template match="version" mode="toc">
    <li><a><xsl:attribute name="href">#<xsl:value-of select="major"/>.<xsl:value-of select="release"/></xsl:attribute>Version <xsl:value-of select="major"/>.<xsl:value-of select="release"/></a> &nbsp; [<xsl:value-of select="date"/>]
    <xsl:if test="changes[text()!='']">- <xsl:value-of select="changes"/></xsl:if></li>
  </xsl:template>

  <xsl:template match="version" mode="release">
    <h3><a><xsl:attribute name="name"><xsl:value-of select="major"/>.<xsl:value-of select="release"/></xsl:attribute></a>Version <xsl:value-of select="major"/>.<xsl:value-of select="release"/></h3>
Version <xsl:value-of select="major"/>.<xsl:value-of select="release"/> has been released on 
    <xsl:value-of select="Util:getMonth(date)" xmlns:Util="xalan://ccl.util.Util"/>
    <xsl:text> </xsl:text>
    <xsl:value-of select="Util:itoa(Util:atoi(substring(date, 9)))" xmlns:Util="xalan://ccl.util.Util"/>
    <xsl:text>, </xsl:text>
    <xsl:value-of select="substring(date, 1, 4)"/>
    <xsl:apply-templates select="new"/>
    <xsl:apply-templates select="bugs"/>
  </xsl:template>

</xsl:stylesheet>
